import asyncio
from datetime import datetime

from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
    RpaTagDTO,
    RpaTagEnum,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from rich.console import Console
import re
from pywinauto.keyboard import send_keys
from worker_automate_hub.utils.util import kill_all_emsys, login_emsys
import warnings
from pywinauto.application import Application
from worker_automate_hub.api.client import get_config_by_name, get_status_cte_emsys
from worker_automate_hub.utils.util import (
    set_variable,
    type_text_into_field,
    worker_sleep,
)
from pywinauto_recorder.player import set_combobox

from datetime import timedelta
import pyautogui
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.utils_nfe_entrada import EMSys

emsys = EMSys()

console = Console()
pyautogui.PAUSE = 0.5
pyautogui.FAILSAFE = False


async def set_tipo_pagamento_combobox(app: Application, window_title: str):
    try:
        app = Application(backend="uia").connect(process=app.process)
        main_window = app.top_window()
        main_window = main_window.child_window(
            title="Conhecimento de Frete", found_index=0
        )
        janelaPagamento = main_window.child_window(title="tsPagamento", found_index=0)
        janelaPagamento.ComboBox.select("BANCO DO BRASIL BOLETO")

    except Exception as e:
        console.print(f"Erro ao conectar a janela {window_title}")
        raise Exception(f"Erro ao conectar a janela: {e}")


def calcular_vencimento(data_emissao_str):
    data_emissao = datetime.strptime(data_emissao_str, "%d/%m/%Y")

    if data_emissao.day <= 15:
        mes_vencimento = data_emissao.month + 1
        ano_vencimento = data_emissao.year
        if mes_vencimento > 12:
            mes_vencimento = 1
            ano_vencimento += 1
        vencimento = datetime(ano_vencimento, mes_vencimento, 1)
    else:
        mes_vencimento = data_emissao.month + 1
        ano_vencimento = data_emissao.year
        if mes_vencimento > 12:
            mes_vencimento = 1
            ano_vencimento += 1
        vencimento = datetime(ano_vencimento, mes_vencimento, 16)

    return vencimento.strftime("%d/%m/%Y")


async def entrada_cte_1353(task: RpaProcessoEntradaDTO):
    try:
        config = await get_config_by_name("login_emsys")
        console.print(task)

        cte = task.configEntrada

        multiplicador_timeout = int(float(task.sistemas[0].timeout))
        set_variable("timeout_multiplicador", multiplicador_timeout)

        # Fecha a instancia do emsys - caso esteja aberta
        await kill_all_emsys()

        # Verifica se o CTE ja foi lançado
        console.print("\nVerifica se o CTE ja foi lançado...")
        nf_chave_acesso = int(cte.get("chaveCte"))
        status_nf_emsys = await get_status_cte_emsys(nf_chave_acesso)
        if status_nf_emsys.get("status") == "Lançada":
            console.print(
                "\\CTE ja lançado, processo finalizado...", style="bold green"
            )
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="CTE já lançado",
                status=RpaHistoricoStatusEnum.Descartado,
            )
        else:
            console.print("\\CTE não lançado, iniciando o processo...")

        app = Application(backend="win32").start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings(
            "ignore",
            category=UserWarning,
            message="32-bit application should be automated using 32-bit Python",
        )

        console.print("\nEMSys iniciando...", style="bold green")
        await worker_sleep(5)

        return_login = await login_emsys(config.conConfiguracao, app, task)
        if return_login.sucesso:
            type_text_into_field(
                "Conhecimento de Frete", app["TFrmMenuPrincipal"]["Edit"], True, "50"
            )
            pyautogui.press("enter")
            await worker_sleep(1)
            pyautogui.press("enter")
            await worker_sleep(5)

            console.print(
                "\nPesquisa: 'Conhecimento de Frete' realizada com sucesso",
                style="bold green",
            )
            await worker_sleep(6)

            console.print("conhecimento frete window")
            type_text_into_field(
                task.configEntrada.get("numeroCte"),
                app["TFrmConhecimentoNotaNew"]["Edit26"],
                False,
                "0",
            )
            await worker_sleep(4)

            type_text_into_field(
                task.configEntrada.get("serieCte"),
                app["TFrmConhecimentoNotaNew"]["Edit24"],
                False,
                "0",
            )
            await worker_sleep(5)

            type_text_into_field(
                task.configEntrada.get("cnpjEmitente"),
                app["TFrmConhecimentoNotaNew"]["Edit9"],
                False,
                "0",
            )

            await worker_sleep(5)
            try:
                app["TFrmConhecimentoNotaNew"]["Edit21"].set_focus()
                app["TFrmConhecimentoNotaNew"]["Edit21"].set_focus()
            except Exception as error:
                console.print(
                    "Não foi possivel focar no campo dataEmissao, clicando na coordenada (903, 417)"
                )
                pyautogui.click(x=903, y=417)
            send_keys(task.configEntrada.get("dataEmissao").replace("/", ""))

            await worker_sleep(4)

            if app["TFrmConhecimentoNotaNew"]["Edit9"].window_text() == "":
                type_text_into_field(
                    task.configEntrada.get("cnpjEmitente"),
                    app["TFrmConhecimentoNotaNew"]["Edit9"],
                    False,
                    "0",
                )
                await worker_sleep(2)

            data_entrada = datetime.now().strftime("%d/%m/%Y")
            try:
                app["TFrmConhecimentoNotaNew"]["Edit20"].set_focus()
                app["TFrmConhecimentoNotaNew"]["Edit20"].set_focus()
            except Exception as error:
                console.print(
                    "Não foi possivel focar no campo dataEntrada, clicando na coordenada (1096, 419)"
                )
                pyautogui.click(x=1096, y=419)
            app["TFrmConhecimentoNotaNew"]["Edit20"].type_keys(
                data_entrada.replace("/", "")
            )
            await worker_sleep(5)

            try:
                app["TFrmConhecimentoNotaNew"]["ComboBox4"].select(
                    "CONHECIMENTO DE FRETE - CTE"
                )
                await worker_sleep(2)
            except Exception as error:
                try:
                    app["TFrmConhecimentoNotaNew"]["ComboBox4"].select(
                        "CONHECIMENTO DE FRETE ELETRONICO - CTe"
                    )
                    await worker_sleep(2)
                except Exception as final_error:
                    console.print(
                        "Nenhuma das opções foi encontrada em (Modelo de Documento)."
                    )

            app["TFrmConhecimentoNotaNew"]["ComboBox3"].select(1)
            await worker_sleep(2)
            app["TFrmConhecimentoNotaNew"]["ComboBox5"].select(2)
            await worker_sleep(2)
            app["TFrmConhecimentoNotaNew"]["ComboBox2"].select("9 - Outros")

            await emsys.verify_warning_and_error("Pesquisa", "&Cancelar")
            type_text_into_field(
                task.configEntrada.get("chaveCte"),
                app["TFrmConhecimentoNotaNew"]["Edit10"],
                False,
                "0",
            )

            nomeEmitente = task.configEntrada.get("nomeEmitente").upper()
            tipo_despesa = "359"

            if "ES SILVEIRA" in nomeEmitente:
                tipo_despesa = "358"

            type_text_into_field(
                tipo_despesa, app["TFrmConhecimentoNotaNew"]["Edit4"], False, "3"
            )
            await worker_sleep(2)

            send_keys("{TAB 3}{DOWN}{ENTER}")
            await worker_sleep(5)
            valorFrete = task.configEntrada.get("valorFrete").replace(".", ",")
            type_text_into_field(
                valorFrete,
                app["TFrmConhecimentoNotaNew"]["Edit21"],
                False,
                "0",
            )
            await worker_sleep(2)

            type_text_into_field(
                valorFrete,
                app["TFrmConhecimentoNotaNew"]["Edit15"],
                False,
                "0",
            )
            await worker_sleep(2)

            send_keys("{TAB 11}{DOWN}{ENTER}")
            await worker_sleep(5)

            pyautogui.click(x=1293, y=537)
            await worker_sleep(2)

            await set_tipo_pagamento_combobox(app, "CONHECIMENTO DE FRETE")
            await worker_sleep(2)

            dataVencimento = calcular_vencimento(task.configEntrada.get("dataEmissao"))

            send_keys("{TAB}" + dataVencimento)
            await worker_sleep(2)

            pyautogui.click(x=1081, y=404)
            await worker_sleep(2)

            pyautogui.click(x=1261, y=402)
            await worker_sleep(2)

            await emsys.verify_warning_and_error("Informação", "&Ok")

            await worker_sleep(5)
            pyautogui.click(x=584, y=323)
            await worker_sleep(20)

            try:
                # Verifica se o CTE ja foi lançado
                console.print("\nVerifica se o CTE ja foi lançado...")
                nf_chave_acesso = int(cte.get("chaveCte"))
                status_nf_emsys = await get_status_cte_emsys(nf_chave_acesso)
                if status_nf_emsys.get("status") == "Lançada":
                    await worker_sleep(2)
                    return RpaRetornoProcessoDTO(
                        sucesso=True,
                        retorno="CTE incluso com sucesso!",
                        status=RpaHistoricoStatusEnum.Sucesso,
                    )
                else:
                    status_retorno = status_nf_emsys.get("status")
                    await worker_sleep(2)
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Ocorreu uma falha ao tentar lançar o CTE, status retornado : {status_retorno}",
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                    )
            except Exception as error:
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Erro ao incluir CTE, erro: {error}",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                )

        else:
            logger.info(f"\nError Message: {return_login.retorno}")
            console.print(f"\nError Message: {return_login.retorno}", style="bold red")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro ao efetuar login no EMsys, erro {return_login.retorno}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )

    except Exception as error:
        console.print(f"Erro ao executar a função error: {error}")
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=f"Erro ao incluir CTE, erro: {error}",
            status=RpaHistoricoStatusEnum.Falha,
            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
        )



import asyncio
import getpass
import os
import warnings
import time
import uuid
import win32clipboard
import difflib
import re
import pyautogui
import pytesseract
from PIL import Image, ImageEnhance
from datetime import datetime
from pywinauto.application import Application
from pywinauto_recorder.player import set_combobox
from pywinauto.keyboard import send_keys
from rich.console import Console
from worker_automate_hub.api.ahead_service import save_xml_to_downloads
from worker_automate_hub.api.client import (
    get_config_by_name,
    get_status_nf_emsys,
)
from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
    RpaTagDTO,
    RpaTagEnum,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import (
    delete_xml,
    error_after_xml_imported,
    get_xml,
    import_nfe,
    incluir_registro,
    is_window_open,
    is_window_open_by_class,
    itens_not_found_supplier,
    kill_all_emsys,
    login_emsys,
    select_documento_type,
    set_variable,
    type_text_into_field,
    warnings_after_xml_imported,
    worker_sleep,
    check_nota_importada,
    carregamento_import_xml,
    errors_generate_after_import,
)

pyautogui.PAUSE = 0.5
pyautogui.FAILSAFE = False
console = Console()


async def entrada_de_notas_7139(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    """
    Processo que relazia entrada de notas no ERP EMSys(Linx).

    """
    try:
        # Get config from BOF
        config = await get_config_by_name("login_emsys")
        console.print(task)

        # Seta config entrada na var nota para melhor entendimento
        nota = task.configEntrada
        multiplicador_timeout = int(float(task.sistemas[0].timeout))
        set_variable("timeout_multiplicador", multiplicador_timeout)

        # Fecha a instancia do emsys - caso esteja aberta
        await kill_all_emsys()

        # Download XML
        console.log("Realizando o download do XML..\n")
        await save_xml_to_downloads(nota["nfe"])

        app = Application(backend="win32").start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings(
            "ignore",
            category=UserWarning,
            message="32-bit application should be automated using 32-bit Python",
        )
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config.conConfiguracao, app, task)

        if return_login.sucesso == True:
            type_text_into_field(
                "Nota Fiscal de Entrada", app["TFrmMenuPrincipal"]["Edit"], True, "50"
            )
            pyautogui.press("enter")
            await worker_sleep(2)
            pyautogui.press("enter")
            console.print(
                f"\nPesquisa: 'Nota Fiscal de Entrada' realizada com sucesso",
                style="bold green",
            )
        else:
            logger.info(f"\nError Message: {return_login.retorno}")
            console.print(f"\nError Message: {return_login.retorno}", style="bold red")
            return return_login

        await worker_sleep(6)
        # Procura campo documento
        console.print("Navegando pela Janela de Nota Fiscal de Entrada...\n")
        document_type = await select_documento_type(
            "NOTA FISCAL DE ENTRADA ELETRONICA - DANFE"
        )
        if document_type.sucesso == True:
            console.log(document_type.retorno, style="bold green")
        else:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=document_type.retorno,
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        await worker_sleep(4)

        # Clica em 'Importar-Nfe'
        imported_nfe = await import_nfe()
        if imported_nfe.sucesso == True:
            console.log(imported_nfe.retorno, style="bold green")
        else:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=imported_nfe.retorno,
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        await worker_sleep(5)

        await get_xml(nota.get("nfe"))
        await worker_sleep(3)

        # VERIFICANDO A EXISTENCIA DE WARNINGS
        await worker_sleep(4)

        warning_pop_up = await is_window_open("Warning")
        if warning_pop_up["IsOpened"] == True:
            warning_work = await warnings_after_xml_imported()
            if warning_work.sucesso == True:
                console.log(warning_work.retorno, style="bold green")
            else:
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=warning_work.retorno,
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )

        # VERIFICANDO A EXISTENCIA DE ERRO
        erro_pop_up = await is_window_open("Erro")
        if erro_pop_up["IsOpened"] == True:
            error_work = await error_after_xml_imported()
            return RpaRetornoProcessoDTO(
                sucesso=error_work.sucesso,
                retorno=error_work.retorno,
                status=error_work.status,
                tags=error_work.tags
            )

        app = Application().connect(
            title="Informações para importação da Nota Fiscal Eletrônica"
        )
        main_window = app["Informações para importação da Nota Fiscal Eletrônica"]

        # INTERAGINDO COM A DATA DE ENTRADA
        await worker_sleep(2)
        try:
            recebimento_fisico = nota.get("recebimentoFisico", None)
            if recebimento_fisico:
                recebimento_fisico = nota["recebimentoFisico"].split(" ")
                pyautogui.write(recebimento_fisico[0])
                await worker_sleep(2)
        except:
            console.print(
                f"A chave recebimentoFisico não está presente na config de entrada...\n"
            )


        # INTERAGINDO COM A NATUREZA DA OPERACAO
        cfop = int(nota.get("cfop"))
        console.print(f"Inserindo a informação da CFOP, caso se aplique {cfop} ...\n")
        if cfop == 5104 or str(cfop).startswith("51"):
            combo_box_natureza_operacao = main_window.child_window(
                class_name="TDBIComboBox", found_index=0
            )
            combo_box_natureza_operacao.click()

            await worker_sleep(3)
            set_combobox("||List", "1102-COMPRA DE MERCADORIA ADQ. TERCEIROS - 1.102")
            await worker_sleep(3)
        elif cfop == 6102 or str(cfop).startswith("61"):
            combo_box_natureza_operacao = main_window.child_window(
                class_name="TDBIComboBox", found_index=0
            )
            combo_box_natureza_operacao.click()

            await worker_sleep(3)
            set_combobox("||List", "2102-COMPRA DE MERCADORIAS SEM DIFAL - 2.102")
            await worker_sleep(3)
        else:
            console.print(
                "Erro mapeado, CFOP diferente de 6102 ou 5104/51, necessario ação manual ou ajuste no robo...\n"
            )
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro mapeado, CFOP diferente de 5655 ou 56, necessario ação manual ou ajuste no robo",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)]
            )

        try:
            console.print(f"Trabalhando com itens com multiplas referencias.\n")
            itens_nao_semelhantes = []
            # TRABALHANDO COM ITENS QUE POSSUEM MULTIPLAS REFERÊNCIAS
            while True:
                await worker_sleep(3)
                console.print(
                    f"Verificando a existencia de fornecedor com múltiplas referencias...\n"
                )
                itens_by_supplier = await is_window_open_by_class(
                    "TFrmSelecionaItensFornecedor", "TFrmSelecionaItensFornecedor"
                )
                if itens_by_supplier["IsOpened"] == True:
                    new_app = Application(backend="uia").connect(
                        class_name="TFrmSelecionaItensFornecedor"
                    )
                    window = new_app["TFrmSelecionaItensFornecedor"]
                    window.set_focus()
                    await worker_sleep(2)

                    console.print(f"Obtendo item com multiplas referências...\n")
                    console.print(f"Tirando print da janela para realização do OCR...\n")

                    text_captured = False
                    count_while = 0
                    max_attempts = 3
                    item_da_nota = ''

                    while count_while < max_attempts:
                        window_rect = window.rectangle()
                        console.print(f"Area que sera utulizada para o screenshot {window_rect}...\n")
                        screenshot = window.capture_as_image()

                        username = getpass.getuser()
                        short_uuid = str(uuid.uuid4()).replace('-', '')[:6]
                        path_to_png = f"C:\\Users\\{username}\\Downloads\\{short_uuid}.png"
                        screenshot.save(path_to_png)
                        while not os.path.exists(path_to_png) or os.path.getsize(path_to_png) == 0:
                            time.sleep(0.1)
                        console.print(f"Print salvo em {path_to_png}...\n")

                        await worker_sleep(2)
                        console.print("Preparando a imagem para maior resolução e assertividade no OCR...\n")
                        image = Image.open(path_to_png)
                        image = image.convert("L")
                        enhancer = ImageEnhance.Contrast(image)
                        image = enhancer.enhance(2.0)
                        image.save(path_to_png)
                        console.print("Imagem preparada com sucesso...\n")

                        console.print(f"Imagem antes do OCR: {image}\n")
                        console.print(f"Dimensões da imagem: {image.size}\n")

                        console.print("Realizando OCR...\n")
                        captured_text = pytesseract.image_to_string(image)
                        console.print(f"Texto Full capturado {captured_text}, tentando obter o item da nota...\n")

                        match = re.search(r"Item da Nota:\s*(.*)\s*", captured_text)
                        if os.path.exists(path_to_png):
                            os.remove(path_to_png)
                            console.print(f"Imagem apagada com sucesso do diretorio {path_to_png}... \n")
                        else:
                            console.print(f"Imagem não encontrada para realização do OCR... \n")

                        console.print(f"Texto extraido do RegEx: {match}... \n")
                        if match:
                            item_da_nota = match.group(1).strip()
                            console.print(f"Item da Nota capturado: {item_da_nota}... \n")
                            text_captured = True
                            break
                        else:
                            if match:
                                item_da_nota = match.group(1).strip()
                                console.print(f"Item da Nota capturado: {item_da_nota}... \n")
                                text_captured = True
                                break
                            else:
                                match = re.search(r"Item da (Nota|Nata|N0ta)\s*(.*)\s*", captured_text)
                                if match:
                                    item_da_nota = match.group(1).strip()
                                    console.print(f"Item da Nota capturado: {item_da_nota}... \n")
                                    text_captured = True
                                    break
                                else:
                                    console.print(f"Tentativa {count_while + 1} de {max_attempts} falhou. Tentando novamente...\n")
                                    count_while += 1

                    if not text_captured:
                        return RpaRetornoProcessoDTO(
                            sucesso=False,
                            retorno="Quantidade de tentativa atingida (3), não foi possivel capturar o item da nota com multiplas referencias para andamento no processo",
                            status=RpaHistoricoStatusEnum.Falha,
                            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                        )

                    console.print(
                        f"Interagindo com os multiplos itens e detectando qual ira ser selecionado para andamento no processo... \n"
                    )
                    copied_codes_list = []
                    count = 0
                    last_copied = None

                    while True:
                        if count == 0:
                            send_keys("^({HOME})")
                        else:
                            send_keys("{DOWN 1}")

                        send_keys("^c")
                        win32clipboard.OpenClipboard()
                        window_message = win32clipboard.GetClipboardData().strip()
                        win32clipboard.CloseClipboard()

                        console.print(f"Linha copiada: {window_message}... \n")

                        if last_copied is not None and window_message == last_copied:
                            break

                        copied_codes_list.append(
                            {"linha": count, "conteudo": window_message}
                        )

                        last_copied = window_message
                        count = +1

                    console.print(f"Valores copiados {copied_codes_list}... \n")

                    extracted_items = {}

                    for entry in copied_codes_list:
                        content = entry["conteudo"]
                        lines = content.split("\r\n")
                        for line in lines:
                            items = line.split("\t")
                            if len(items) >= 3:
                                cod_item = items[0].strip()
                                descricao = items[1].strip()
                                cod_barra = items[2].strip()

                                extracted_items[cod_item] = {
                                    "Descrição": descricao,
                                    "Código de Barra": cod_barra,
                                    "Linha": entry["linha"],
                                }

                    best_match = None
                    highest_ratio = 0.0

                    for cod_item, info in extracted_items.items():
                        descricao = info["Descrição"]
                        similarity_ratio = difflib.SequenceMatcher(
                            None, item_da_nota, descricao
                        ).ratio()

                        if similarity_ratio > highest_ratio:
                            highest_ratio = similarity_ratio
                            best_match = {
                                "Cod Item": cod_item,
                                "Descrição": descricao,
                                "Código de Barra": info["Código de Barra"],
                                "Linha": info["Linha"],
                                "Similaridade": similarity_ratio,
                            }

                    if best_match and highest_ratio > 0.7:
                        console.print(
                            f"Melhor semelhança encontrada {best_match}, selecionando... \n"
                        )
                        send_keys("^({HOME})")
                        send_keys("{DOWN " + str(best_match["Linha"]) + "}")
                        send_keys("%o")
                        await worker_sleep(2)
                    else:
                        itens_nao_semelhantes.append(
                            {"item": item_da_nota, "match": best_match}
                        )
                        send_keys("%o")
                else:
                    console.print(
                        "Não possui a tela de fornecedor com múltiplas referencias, seguindo com o processo...\n"
                    )
                    break

            if len(itens_nao_semelhantes) > 0:
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Não foi possivel encontrar o item mais proximo ao item da nota com multiplas referencias {itens_nao_semelhantes}",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )
        except Exception as error:
            console.print("Erro durante a trativa de multiplas referencias, erro : {error}")

        # INTERAGINDO COM O CAMPO ALMOXARIFADO
        filialEmpresaOrigem = nota.get("filialEmpresaOrigem")
        console.print(
            f"Inserindo a informação do Almoxarifado {filialEmpresaOrigem} ...\n"
        )
        # task_bar_toast("Teste toast bar", f"Inserindo a informação do Almoxarifado {filialEmpresaOrigem} ...", 'Worker', 10)
        # show_toast("Teste toast", f"Inserindo a informação do Almoxarifado {filialEmpresaOrigem} ...")
        try:
            new_app = Application(backend="uia").connect(
                title="Informações para importação da Nota Fiscal Eletrônica"
            )
            window = new_app["Informações para importação da Nota Fiscal Eletrônica"]
            edit = window.child_window(
                class_name="TDBIEditCode", found_index=3, control_type="Edit"
            )
            valor_almoxarifado = filialEmpresaOrigem + "50"
            edit.set_edit_text(valor_almoxarifado)
            edit.type_keys("{TAB}")
        except Exception as e:
            console.print(f"Erro ao iterar itens de almoxarifado: {e}")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro ao iterar itens de almoxarifado: {e}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )
        
        window.child_window(title="Manter Natureza de Operação selecionada", control_type="CheckBox").click()

        await worker_sleep(2)
        console.print("Clicando em OK... \n")
        try:
            btn_ok = main_window.child_window(title="Ok")
            btn_ok.click()
        except:
            btn_ok = main_window.child_window(title="&Ok")
            btn_ok.click()
        await worker_sleep(6)

        max_attempts = 3
        i = 0
        while i < max_attempts:
            console.print("Clicando no botão de OK...\n")
            try:
                try:
                    btn_ok = main_window.child_window(title="Ok")
                    btn_ok.click()
                except:
                    btn_ok = main_window.child_window(title="&Ok")
                    btn_ok.click()
            except:
                console.print("Não foi possivel clicar no Botão OK... \n")

            await worker_sleep(3)

            console.print(
                "Verificando a existencia da tela Informações para importação da Nota Fiscal Eletrônica...\n"
            )

            try:
                informacao_nf_eletronica = await is_window_open(
                    "Informações para importação da Nota Fiscal Eletrônica"
                )
                if informacao_nf_eletronica["IsOpened"] == False:
                    console.print(
                        "Tela Informações para importação da Nota Fiscal Eletrônica fechada, seguindo com o processo"
                    )
                    break
            except Exception as e:
                console.print(
                    f"Tela Informações para importação da Nota Fiscal Eletrônica encontrada. Tentativa {i + 1}/{max_attempts}."
                )

            i += 1

        if i == max_attempts:
            return {
                "sucesso": False,
                "retorno": f"Número máximo de tentativas atingido, Não foi possivel finalizar os trabalhos na tela de Informações para importação da Nota Fiscal Eletrônica",
            }
        
        await worker_sleep(2)
        waiting_for_delay = await carregamento_import_xml()
        if waiting_for_delay.sucesso:
            console.print(waiting_for_delay.retorno)
        else:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=waiting_for_delay.retorno,
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        try:
            console.print("Verificando itens não localizados ou NCM...\n")
            itens_by_supplier = await is_window_open_by_class("TFrmAguarde", "TMessageForm")

            if itens_by_supplier["IsOpened"] == True:
                itens_by_supplier_work = await itens_not_found_supplier(nota.get("nfe"))

                if not itens_by_supplier_work.sucesso:
                    return itens_by_supplier_work

        except Exception as error:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Falha ao verificar a existência de POP-UP de itens não localizados: {error}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )
        
        logs_erro = await is_window_open_by_class("TFrmExibeLogErroImportacaoNfe","TFrmExibeLogErroImportacaoNfe")
        if logs_erro["IsOpened"] == True:
            errors_genetared = await errors_generate_after_import(nota.get("nfe"))
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=errors_genetared.retorno,
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        await worker_sleep(6)

        max_attempts = 7
        i = 0
        while i < max_attempts:
            await worker_sleep(2)
            aguarde_aberta = False
            from pywinauto import Desktop

            for window in Desktop(backend="uia").windows():
                if "Aguarde" in window.window_text():
                    aguarde_aberta = True
                    console.print("A janela 'Aguarde' está aberta. Aguardando...\n")
                    break

            i += 1

            if not aguarde_aberta:
                console.print(
                    "A janela 'Aguarde' foi fechada. Continuando para encerramento do processo...\n"
                )
                break

        if i == max_attempts:
            return {
                "sucesso": False,
                "retorno": f"Número máximo de tentativas atingido. A tela para Aguarde não foi encerrada.",
            }
            
        await worker_sleep(2)
        
        try:
            console.print("Verificando itens não localizados ou NCM...\n")
            itens_by_supplier = await is_window_open_by_class("TFrmAguarde", "TMessageForm")

            if itens_by_supplier["IsOpened"] == True:
                itens_by_supplier_work = await itens_not_found_supplier(nota.get("nfe"))

                if not itens_by_supplier_work.sucesso:
                    return itens_by_supplier_work

        except Exception as error:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Falha ao verificar a existência de POP-UP de itens não localizados: {error}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        console.print("Navegando pela Janela de Nota Fiscal de Entrada...\n")
        app = Application().connect(class_name="TFrmNotaFiscalEntrada")
        main_window = app["TFrmNotaFiscalEntrada"]

        main_window.set_focus()
        console.print("Acessando a aba de Pagamentos... \n")
        panel_TPage = main_window.child_window(class_name="TPage", title="Formulario")
        panel_TTabSheet = panel_TPage.child_window(class_name="TcxCustomInnerTreeView")
        panel_TTabSheet.wait("visible")
        panel_TTabSheet.click()
        send_keys("{DOWN " + ("7") + "}")

        panel_TPage = main_window.child_window(class_name="TPage", title="Formulario")
        panel_TTabSheet = panel_TPage.child_window(class_name="TPageControl")

        panel_TabPagamento = panel_TTabSheet.child_window(class_name="TTabSheet")

        panel_TabParcelamento = panel_TTabSheet.child_window(title="Parcelamento")

        tipo_cobranca = panel_TabParcelamento.child_window(
            class_name="TDBIComboBox", found_index=0
        )

        console.print("Verificando o tipo de cobrança selecionado... \n")
        tipo_selecionado = tipo_cobranca.window_text()
        if "boleto" in tipo_selecionado.lower() or 'carteira' in tipo_selecionado.lower():
            console.print(f"Tipo de cobrança corretamente selecionado {tipo_selecionado}... \n")
        else:
            console.print(f"Tipo de cobrança não foi selecionado corretamente, interagindo com o campo para selecionar o campo corretamente... \n")
            tipo_cobranca.click()
            try:
                set_combobox("||List", "BANCO DO BRASIL BOLETO")
            except:
                set_combobox("||List", "CARTEIRA")
        
        await worker_sleep(2)

        # Inclui registro
        console.print(f"Incluindo registro...\n")
        await worker_sleep(6)
        try:
            ASSETS_PATH = "assets"
            inserir_registro = pyautogui.locateOnScreen(
                ASSETS_PATH + "\\entrada_notas\\IncluirRegistro.png", confidence=0.8
            )
            pyautogui.click(inserir_registro)
        except Exception as e:
            console.print(
                f"Não foi possivel incluir o registro utilizando reconhecimento de imagem, Error: {e}...\n tentando inserir via posição...\n"
            )
            await incluir_registro()
        

        await worker_sleep(6)
        nf_imported = await check_nota_importada(nota.get("nfe"))
        if nf_imported.sucesso == True:
            await worker_sleep(3)
            console.print("\nVerifica se a nota ja foi lançada...")
            nf_chave_acesso = int(nota.get("nfe"))
            status_nf_emsys = await get_status_nf_emsys(nf_chave_acesso)
            if status_nf_emsys.get("status") == "Lançada":
                console.print("\nNota lançada com sucesso, processo finalizado...", style="bold green")
                return RpaRetornoProcessoDTO(
                    sucesso=True,
                    retorno="Nota Lançada com sucesso!",
                    status=RpaHistoricoStatusEnum.Sucesso,
                )
            else:
                console.print("Erro ao lançar nota", style="bold red")
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Pop-up nota incluida encontrada, porém nota encontrada como 'já lançada' trazendo as seguintes informações: {nf_imported.retorno} - {error_work}",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)]
                )
        else:
            console.print("Erro ao lançar nota", style="bold red")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro ao lançar nota, erro: {nf_imported.retorno}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

    except Exception as ex:
        observacao = f"Erro Processo Entrada de Notas: {str(ex)}"
        logger.error(observacao)
        console.print(observacao, style="bold red")
        return {"sucesso": False, "retorno": observacao}

    finally:
        # Deleta o xml
        await delete_xml(nota.get("nfe"))



import asyncio
import getpass
from datetime import datetime
import pyperclip
from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
    RpaTagDTO,
    RpaTagEnum,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from rich.console import Console
import re
import io
import os
import time
import json
import pandas as pd
from pathlib import Path

from pywinauto.keyboard import send_keys
from worker_automate_hub.utils.util import login_emsys
import warnings
from pywinauto.application import Application
from worker_automate_hub.api.client import (
    get_config_by_name,
    get_status_nf_emsys,
    send_file,
)
from worker_automate_hub.api.ahead_service import save_xml_to_downloads
from worker_automate_hub.utils.util import (
    kill_all_emsys,
    delete_xml,
    set_variable,
    type_text_into_field,
    worker_sleep,
)
from pywinauto_recorder.player import set_combobox
from worker_automate_hub.api.datalake_service import send_file_to_datalake

from datetime import timedelta
import pyautogui
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.utils_nfe_entrada import EMSys

ASSETS_BASE_PATH = "assets"
emsys = EMSys()

console = Console()
pyautogui.PAUSE = 0.5
pyautogui.FAILSAFE = False


async def wait_aguarde_window_closed(app, timeout=180):
    console.print("Verificando existencia de aguarde...")
    start_time = time.time()

    while time.time() - start_time < timeout:
        try:
            janela_topo = app.top_window()
            titulo = janela_topo.window_text()
            console.print(f"Título da janela no topo: {titulo}")
        except Exception as e:
            console.print(f"Erro ao obter janela do topo: {e}", style="bold red")
            titulo = ""

        # Verifica se há alertas de aviso ou erro e clica no botão OK
        await emsys.verify_warning_and_error("Aviso", "&Ok")

        # Aguarda um pouco antes de verificar de novo
        await worker_sleep(2)

        if "Relatório Extrato Bancário" in titulo:
            console.log("Fim de aguardando...")
            return
        else:
            console.log("Aguardando...")

    console.log("Timeout esperando a janela Aguarde...")


async def extracao_fechamento_emsys(
    task: RpaProcessoEntradaDTO,
) -> RpaRetornoProcessoDTO:
    try:
        config = await get_config_by_name("login_emsys")
        periodo_inicial = task.configEntrada["periodoInicial"]
        periodo_final = task.configEntrada["periodoFinal"]
        historico_id = task.historico_id
        await kill_all_emsys()

        app = Application(backend="win32").start("C:\\Rezende\\EMSys3\\EMSys3_35.exe")
        warnings.filterwarnings(
            "ignore",
            category=UserWarning,
            message="32-bit application should be automated using 32-bit Python",
        )
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(
            config.conConfiguracao, app, task, filial_origem=1
        )

        if return_login.sucesso == True:
            type_text_into_field(
                "Rel. Extrato Bancário", app["TFrmMenuPrincipal"]["Edit"], True, "50"
            )
            pyautogui.press("enter")
            await worker_sleep(2)
            pyautogui.press("enter")

        else:
            logger.info(f"\nError Message: {return_login.retorno}")
            console.print(f"\nError Message: {return_login.retorno}", style="bold red")
            return return_login

        await worker_sleep(6)

        ##### Janela Extrato Bancário #####

        # Clicar na lupa
        pyautogui.click(x=1077, y=351)

        await worker_sleep(2)

        # Armazenar os Mneumonico
        linhas = []
        vistos = set()
        ultima_linha = ""
        repeticoes = 0
        limite_repeticoes = 3
        max_linhas = 1000

        console.print("Iniciando leitura dos mnemônicos..")
        # Clicar na primeira linha
        pyautogui.click(x=698, y=467)

        await worker_sleep(2)

        # Percorrer todas as linhas
        for _ in range(max_linhas):
            send_keys("^c")
            await asyncio.sleep(0.5)
            linha = pyperclip.paste().strip()

            if not linha:
                print("[Aviso] Linha vazia detectada. Pulando...")
                send_keys("{DOWN}")
                await asyncio.sleep(0.3)
                continue

            if linha == ultima_linha:
                repeticoes += 1
                if repeticoes >= limite_repeticoes:
                    print("Fim detectado: linha repetida várias vezes.")
                    break
            else:
                repeticoes = 0

            if linha not in vistos:
                print(f"[{len(linhas)+1}] {linha}")
                linhas.append(linha)
                vistos.add(linha)

            ultima_linha = linha
            send_keys("{DOWN}")
            await asyncio.sleep(0.3)

        console.print("Números mnemônicos armazenados")
        # Extrair apenas o mnemônico da linha de dados após o cabeçalho
        mnemonicos_detalhados = []
        for linha in linhas:
            if "Mnemônico" in linha:
                partes = linha.split("\r\n")
                if len(partes) > 1:
                    dados = partes[1].split("\t")
                    if len(dados) >= 4:
                        mnemonico = dados[0].strip()
                        agencia = dados[1].strip()
                        conta = dados[2].strip()
                        banco = dados[3].strip()

                        mnemonicos_detalhados.append(
                            {
                                "mnemonico": mnemonico,
                                "agencia": agencia,
                                "conta": conta,
                                "banco": banco,
                            }
                        )

        ##### Janela  Buscar Mneumônico
        pyautogui.click(x=1196, y=650)

        saldos = []
        # Extrair apenas o mnemônico da linha de dados após o cabeçalho
        mnemonicos_detalhados = []
        clicou_conciliados = False
        inserir_periodo = False
        for linha in linhas:
            caminho_excel = ""
            filename = ""
            await worker_sleep(1)
            ##### Janela Relatório Extrato Bancário #####
            app = Application(backend="win32").connect(
                class_name="TFrmRelExtratoBancario", found_index=0
            )
            main_window = app["TFrmRelExtratoBancario"]

            # Inserir Mnemonico
            console.log("Iniciando  Baixa do XLS")
            if "Mnemônico" in linha:
                partes = linha.split("\r\n")
                if len(partes) > 1:
                    dados = partes[1].split("\t")
                    if len(dados) >= 4:
                        mnemonico = dados[0].strip()
                        agencia = dados[1].strip()
                        conta = dados[2].strip()
                        banco = dados[3].strip()

                        mnemonicos_detalhados.append(
                            {
                                "mnemonico": mnemonico,
                                "agencia": agencia,
                                "conta": conta,
                                "banco": banco,
                            }
                        )
            conta_movimento = f"{banco}/ AG {agencia}/ Conta {conta}"
            input_mnemonico = main_window.child_window(
                class_name="TDBIEditCode", found_index=0
            )
            console.print(f"Inserindo Mnemônico: {mnemonico}")
            type_text_into_field(
                text=mnemonico,
                field=input_mnemonico,
                empty_before=True,
                chars_to_empty="10",
            )
            await worker_sleep(1)

            send_keys("{TAB}")

            await worker_sleep(1)
            if not inserir_periodo:
                console.print("Inserindo perído inicial")
                # Inserir período inicial
                input_data_inicial = main_window.child_window(
                    class_name="TDBIEditDate", found_index=3
                )

                type_text_into_field(
                    text=periodo_inicial,
                    field=input_data_inicial,
                    empty_before=True,
                    chars_to_empty="10",
                )
                await worker_sleep(1)

                console.print("Inserindo perído final")
                input_data_final = main_window.child_window(
                    class_name="TDBIEditDate", found_index=2
                )

                type_text_into_field(
                    text=periodo_final,
                    field=input_data_final,
                    empty_before=True,
                    chars_to_empty="10",
                )
                inserir_periodo = True
                await worker_sleep(1)
            console.print("Marcando movimentos conciliados")
            # Marcar movimentos conciliados
            if not clicou_conciliados:
                chk_mov_conciliados = main_window.child_window(
                    class_name="TDBICheckBox", found_index=2
                ).click()

                await worker_sleep(2)

                console.print("Selecionar tipo Excel")
                # Selecionar Excel
                slc_excel = main_window.child_window(
                    class_name="TDBIComboBoxValues", found_index=0
                ).select("Excel")
                clicou_conciliados = True

            await worker_sleep(3)

            # Clicar em gerar relatório
            console.print("Clicar em gerar relatório")            
            btn_gerar_relatorio = main_window.child_window(class_name="TBitBtn", found_index=0).click_input()
            
            await worker_sleep(7)
            console.print("Verificar se existem dados")

            try:
                nao_existe_dados = fr"{ASSETS_BASE_PATH}\extracao_fechamento_emsys\nao_existem_dados.png"

                for tentativa in range(3):
                    console.print(f"Tentativa {tentativa + 1}: Clicar em 'Gerar Relatório'")
                 
                    btn_gerar_relatorio = main_window.child_window(class_name="TBitBtn", found_index=0)
                    if btn_gerar_relatorio.exists(timeout=5):
                        btn_gerar_relatorio.click_input()
                        await worker_sleep(5)

                        # Verifica se apareceu a imagem de "sem dados"
                        localizacao = pyautogui.locateOnScreen(
                            nao_existe_dados, confidence=0.9
                        )
                        if localizacao:
                            console.print(f"Nenhum dado para {mnemonico}")
                            valor = "0,00"

                            try:
                                app_sem_dados = Application(backend="win32").connect(
                                    title="Information", found_index=0
                                )
                                window_sem_dados = app_sem_dados["Information"]
                                botao_ok = window_sem_dados.child_window(
                                    class_name="TButton", found_index=0
                                ).click()
                            except Exception as e:
                                console.print(
                                    f"Erro ao fechar janela 'Information': {e}",
                                    style="bold red",
                                )

                            await worker_sleep(5)
                            break  

                        # Verifica se apareceu a janela "Salvar para arquivo"
                        try:
                            app_salvar = Application(backend="win32").connect(
                                title="Salvar para arquivo", found_index=0
                            )
                            main_window_salvar = app_salvar["Salvar para arquivo"]
                            console.print("Janela 'Salvar para arquivo' encontrada.")
                            break  # Janela encontrada, pode sair do loop
                        except:
                            console.print(
                                "Janela 'Salvar para arquivo' não encontrada.",
                                style="yellow",
                            )

                    else:
                        console.print(
                            "Botão 'Gerar Relatório' não encontrado.", style="bold red"
                        )
                        break
            except:      
                await worker_sleep(5)                   
                ##### Janela Salvar para arquivo #####
                console.print(f"Dados encontrados para {mnemonico}")
                app = Application(backend="win32").connect(
                    title="Salvar para arquivo", found_index=0
                )
                main_window = app["Salvar para arquivo"]
                date_now = datetime.now().strftime("%Y%m%d%H%M%S")
                data_inicial_arquivo = periodo_inicial.replace("/", "")
                data_final_arquivo = periodo_final.replace("/", "")

                # Caminho completo para Downloads
                nome_arquivo = f"C:\\Users\\{getpass.getuser()}\\Downloads\\fechamento_{data_inicial_arquivo}_{data_final_arquivo}_{date_now}.XLS"

                console.print(f"Salvar arquivo: {nome_arquivo}")

                # Inserir nome do arquivo
                console.print(f"Inserir caminho do arquivo: {nome_arquivo}")
                input_nome = main_window.child_window(class_name="Edit", found_index=0)
                type_text_into_field(
                    nome_arquivo, input_nome, empty_before=False, chars_to_empty="0"
                )

                await worker_sleep(2)

                console.print("Salvando o arquivo")
                # Clicar em salvar
                botao_salvar = main_window.child_window(
                    class_name="Button", found_index=0
                )
                botao_salvar.click_input()

                await worker_sleep(6)

                await wait_aguarde_window_closed(app)

                arquivo_path = Path(nome_arquivo)
                # Altera a extensão para .XLS
                caminho_arquivo = arquivo_path.with_suffix(".XLS")
                # Altera a extensão final para .xls minúsculo
                caminho_ajustado = caminho_arquivo.with_suffix(".xls")
                nome_com_extensao = caminho_ajustado.name
                print(nome_com_extensao)
                # Renomeia o arquivo
                os.rename(caminho_arquivo, caminho_ajustado)

                console.print(f"Arquivo renomeado para: {caminho_ajustado}")

                await worker_sleep(1)

                console.print("Extraindo os dados do Excel")

                # Envia o arquivo para o BOF
                df = pd.read_excel(caminho_ajustado, header=None, engine="xlrd")

                # Filtra onde a coluna Q (índice 16) contém "Total Geral"
                mask = (
                    df[16].astype(str).str.contains("Total Geral", case=False, na=False)
                )
                linha_subtot = df[mask].index

                valor = None

                if not linha_subtot.empty:
                    index_subtot = linha_subtot[0]
                    index_verificacao = index_subtot - 2

                    # Regex para aceitar apenas números, vírgulas, pontos e sinal de menos
                    padrao_numerico = re.compile(r"^-?[\d\.\,]+$")

                    while index_verificacao >= 0:
                        valor_bruto = str(df.at[index_verificacao, 23]).strip()

                        # Verifica se não contém letras, apenas caracteres numéricos válidos
                        if padrao_numerico.fullmatch(valor_bruto):
                            valor = (
                                valor_bruto  # Mantém exatamente como está na planilha
                            )
                            break

                        index_verificacao -= 1

                    if valor is not None:
                        console.print(f"Valor encontrado: {valor}")
                    else:
                        console.print(
                            "Nenhum valor numérico puro encontrado nas linhas anteriores."
                        )
                else:
                    console.print("Linha contendo 'Total Geral' não encontrada.")
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Erro ao enviar o arquivo: {e}",
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                    )
            console.print("Adicionando dados no json")
            await worker_sleep(2)
            # Adiciona o item à lista de saldos
            saldos.append(
                {
                    "mnemonico": mnemonico,
                    "contaMovimento": conta_movimento,
                    "valor": valor,
                }
            )

            try:
                # lê o arquivo
                with open(f"{caminho_ajustado}", "rb") as file:
                    file_bytes = io.BytesIO(file.read())

                console.print("Enviar Excel para o BOF")
                # Envia o arquivo para o BOF
                try:
                    await send_file(
                        historico_id,
                        nome_com_extensao,
                        "xls",
                        file_bytes,
                        file_extension="xls",
                    )
                    console.print("Removendo arquivo XLS da pasta downloads")
                    os.remove(f"{caminho_ajustado}")

                except Exception as e:
                    console.print(f"Erro ao enviar o arquivo: {e}", style="bold red")
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Erro ao enviar o arquivo: {e}",
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                    )
            except:
                pass
        console.print("Criando estrutura do Json")
        # Cria estrutura final do JSON
        dados_json = {
            "fechamento": {
                "periodoInicial": periodo_inicial,
                "periodoFinal": periodo_final,
                "dataHora": datetime.now().strftime("%Y-%m-%d %H:%M"),
            },
            "saldos": saldos,
        }

        console.print("Salvando arquivo json")
        # Salva em arquivo
        filename = (
            f"fechamento_{data_inicial_arquivo}_{data_final_arquivo}_{date_now}.json"
        )
        caminho_downloads = os.path.join(os.environ["USERPROFILE"], "Downloads")
        full_path = os.path.join(
            caminho_downloads,
            f"fechamento_{data_inicial_arquivo}_{data_final_arquivo}_{date_now}.json",
        )

        with open(full_path, "w", encoding="utf-8") as f:
            json.dump(dados_json, f, ensure_ascii=False, indent=2)

        print(f"Arquivo JSON salvo em: {full_path}")

        console.print("Enviar arquivo para o Datalake")
        # Envia o JSON para o datalake
        directory = "balancete_contabil/raw"

        with open(full_path, "rb") as file:
            file_bytes = io.BytesIO(file.read())
        try:
            send_file_request = await send_file_to_datalake(
                directory, file_bytes, filename, "json"
            )
            os.remove(full_path)
            return RpaRetornoProcessoDTO(
                sucesso=True,
                retorno=json.dumps(dados_json),
                status=RpaHistoricoStatusEnum.Sucesso,
            )
        except Exception as e:
            console.print(f"Erro ao enviar o arquivo: {e}", style="bold red")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro ao enviar o arquivo: {e}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )

    except Exception as e:
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=f"Erro no processo Extração de fechamento Emsys: {e}",
            status=RpaHistoricoStatusEnum.Falha,
            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
        )
    finally:
        print("fim")



import asyncio
import getpass
from datetime import datetime
from pyautogui import ImageNotFoundException
from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
    RpaTagDTO,
    RpaTagEnum,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from rich.console import Console
import re
import time
import os
import numpy
from pywinauto.keyboard import send_keys
from pywinauto.findwindows import ElementNotFoundError
from worker_automate_hub.utils.util import login_emsys
import warnings
from PIL import ImageFilter, ImageEnhance
from pytesseract import image_to_string
from pywinauto.application import Application
from worker_automate_hub.api.client import get_config_by_name, get_status_cte_emsys
from worker_automate_hub.api.ahead_service import save_xml_to_downloads
from worker_automate_hub.utils.util import (
    kill_all_emsys,
    delete_xml,
    set_variable,
    type_text_into_field,
    worker_sleep,
)
from pywinauto_recorder.player import set_combobox

from datetime import timedelta
import pyautogui
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.utils_nfe_entrada import EMSys

pyautogui.PAUSE = 0.5
console = Console()
emsys = EMSys()

ASSETS_PATH_BASE = "assets"


async def localizar_e_clicar(caminho_imagem, tentativas=50, scroll_pixels=300):
    for tentativa in range(tentativas):
        try:
            pos = pyautogui.locateCenterOnScreen(caminho_imagem, confidence=0.9)
        except:
            print(f"Erro")
            pos = None
        if pos:
            pyautogui.click(pos)
            print(f"Imagem encontrada e clicada na tentativa {tentativa + 1}")
            return True
        else:
            pyautogui.scroll(-scroll_pixels)
            await worker_sleep(0.5)
    print("Não encontrou a imagem após as tentativas.")
    return False


def get_text_from_window(window, relative_coords, value=None):
    try:
        screenshot = window.capture_as_image()
        imagem = screenshot.convert("L")
        imagem = imagem.filter(ImageFilter.SHARPEN)
        imagem = ImageEnhance.Contrast(imagem).enhance(2)
        cropped_screenshot = imagem.crop(relative_coords)
        texto = image_to_string(cropped_screenshot, lang="por")
        return (value.upper() in texto.upper()) if value != None else texto.lower()
    except Exception as error:
        console.print(f"Error: {error}")


async def open_contabil_processes():
    try:
        console.print("Abrindo EMSys Contabil...")
        os.startfile("C:\\Users\\automatehub\\Desktop\\Contabil\\contabil1.lnk")
        await worker_sleep(3)
        os.startfile("C:\\Users\\automatehub\\Desktop\\Contabil\\contabil2.lnk")
        await worker_sleep(60)
        os.startfile("C:\\Users\\automatehub\\Desktop\\Contabil\\contabil3.lnk")
        await worker_sleep(60)
        pyautogui.hotkey("win", "d")
        await worker_sleep(20)
        os.startfile("C:\\Users\\automatehub\\Desktop\\Contabil\\contabil4.lnk")
        await worker_sleep(20)
    except Exception as error:
        console.print(f"Error: {error}")


async def metodo_selecao_origem_especial():

    try:
        app = Application(backend="win32").connect(
            class_name="TFrmIntegrador", found_index=0
        )
        main_window = app["TFrmIntegrador"]

        # clique no combobox
        combobox = main_window.child_window(
            class_name="TcxCheckComboBox", found_index=1
        ).wrapper_object()
        combobox.click_input()

        await worker_sleep(2)
    except:
        # Clica no campo para selecionar a origem
        pyautogui.click(x=653, y=379)


async def integracao_contabil_generica(
    task: RpaProcessoEntradaDTO,
) -> RpaRetornoProcessoDTO:
    try:
        multiplicador_timeout = int(float(task.sistemas[0].timeout))
        set_variable("timeout_multiplicador", multiplicador_timeout)
        await kill_all_emsys()
        await open_contabil_processes()
        config = await get_config_by_name("login_emsys_contabil")

        app = None
        max_attempts = 30
        console.print("Tentando encontrar janela de login...")
        for attempt in range(max_attempts):
            try:
                app = Application(backend="win32").connect(
                    title="Selecione o Usuário para autenticação"
                )
                console.print("Janela encontrada!")
                break
            except:
                console.print("Janela ainda nao encontrada...")
                await worker_sleep(2)
        if not app:
            console.print("Nao foi possivel encontrar a janela de login...")
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Erro durante tentativa localizacao de janelas...",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )
        await emsys.verify_warning_and_error("Erro", "&Ok")
        await worker_sleep(10)
        pyautogui.click(x=1021, y=127)
        console.print("Logando...")
        await emsys.verify_warning_and_error("Erro", "&Ok")
        pyautogui.write(config.conConfiguracao.get("user"))
        pyautogui.press("enter")

        await worker_sleep(4)
        pyautogui.write(config.conConfiguracao.get("pass"))
        pyautogui.press("enter")
        await worker_sleep(10)

        main_window = None
        for attempt in range(max_attempts):
            main_window = Application().connect(title="EMSys [Contabil]")
            main_window = main_window.top_window()
            if main_window.exists():
                console.print("Janela encontrada!")
                break
            console.print("Janela ainda nao encontrada...")
            await worker_sleep(1)

        if not main_window:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Erro durante tentativa localizacao de janelas....",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )

        # Adicionando foco
        try:
            main_window.set_focus()
            console.print(f"Ativando janela: {main_window}")
        except Exception as error:
            console.print(f"Erro ao setar foco na janela: {main_window}")

        await worker_sleep(4)
        console.print("Iniciando interacao a janela de integracao contabil...")
        pyautogui.click(x=566, y=53)
        await worker_sleep(4)
        pyautogui.press("tab", presses=2, interval=1)
        await worker_sleep(4)
        pyautogui.press("enter")
        await worker_sleep(12)
        # pyautogui.press("tab")
        await worker_sleep(3)

        # console.print("Selecionando item do campo origem...")
        uuid_processo = task.uuidProcesso
        caminho_imagem = f"assets\\integracao_contabil\\{uuid_processo}.png"

        await metodo_selecao_origem_especial()
        await localizar_e_clicar(caminho_imagem)  # main_window.set_focus()
        await worker_sleep(5)

        console.print("Preenchendo campo periodo...")
        app = Application(backend="win32").connect(
            class_name="TFrmIntegrador", found_index=0
        )
        main_window = app["TFrmIntegrador"]
        await worker_sleep(4)
        periodo_inicial = task.configEntrada.get("periodoInicial")
        periodo_inicial = periodo_inicial.replace("/", "")
        # clique no combobox
        combobox = main_window.child_window(
            class_name="TRzEditDate", found_index=1
        ).wrapper_object()

        combobox.click_input()  # Clica no campo para focar
        combobox.type_keys("{BACKSPACE 5}")  # Apaga os 5 últimos caracteres

        await worker_sleep(2)

        combobox.type_keys(periodo_inicial)

        # Preencher Período final
        periodo_final = task.configEntrada.get("periodoFinal")
        periodo_final = periodo_final.replace("/", "")
        pyautogui.write(periodo_final)

        # clique no combobox
        combobox = main_window.child_window(
            class_name="TRzEditDate", found_index=0
        ).wrapper_object()

        combobox.click_input()  # Clica no campo para focar
        combobox.type_keys("{BACKSPACE 5}")  # Apaga os 5 últimos caracteres

        await worker_sleep(2)

        combobox.type_keys(periodo_final)  # Digita a nova data

        await worker_sleep(2)

        console.print("Clicando no botao pesquisar...")
        combobox = main_window.child_window(
            class_name="TBitBtn", found_index=3
        ).wrapper_object()
        combobox.click_input()

        await worker_sleep(10)

        console.print("Verificar se existem lotes")
        try:
            # Verifica mensagem sem lote pra integrar
            imagem_alvo = "assets\\integracao_contabil\\sem_lote.png"

            localizacao = pyautogui.locateOnScreen(imagem_alvo, confidence=0.9)

            if localizacao:
                console.print("Imagem sem lote para integrar encontrada!")
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Sem lotes para integrar.",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
                )
            else:
                console.print("Imagem não encontrada.")

        except ImageNotFoundException:
            console.print(
                "Imagem não encontrada (exceção capturada). Tentando clicar no OK."
            )
            try:
                app = Application(backend="win32").connect(
                    class_name="TMsgBox", found_index=0
                )
                main_window = app["TMsgBox"]
                main_window.child_window(
                    class_name="TBitBtn", found_index=0
                ).click_input()
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Sem lotes para integrar.",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
                )
            except Exception as e:
                console.print(f"Não foi possível clicar no botão OK: {e}")

        await worker_sleep(5)

        try:
            imagem_finalizada = "assets\\integracao_contabil\\pesquisa_finalizada.png"

            if not os.path.exists(imagem_finalizada):
                raise FileNotFoundError(f"Imagem não encontrada no caminho: {imagem_finalizada}")

            print("Aguardando a imagem aparecer...")

            localizacao = None
            max_tentativas = 300

            for tentativa in range(1, max_tentativas + 1):
                print(f"Tentativa {tentativa} de {max_tentativas}...")
                try:
                    localizacao = pyautogui.locateOnScreen(imagem_finalizada, confidence=0.85)
                    if localizacao:
                        console.print("Imagem encontrada!")
                        break
                    else:
                        print("Imagem não encontrada ainda.")
                except Exception as e:
                    print(f"Erro ao verificar a imagem: {e}")
                
                await worker_sleep(20)

            if not localizacao:
                console.print("Imagem não foi encontrada após 300 tentativas")
                return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno="Imagem de pesquisa finalizada não encontrada, favor verificar.",
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
                    )
        except Exception as e:
            console.print(f"Erro ao procurar imagem: {e}")

        # Conecta ao aplicativo
        app = Application(backend="win32").connect(title_re=".*Integrador Contábil.*")

        # Janela principal
        dlg = app.window(title_re=".*Integrador Contábil.*")

        # Captura todos os campos do tipo TRzEditNumber
        numeros = [
            e
            for e in dlg.descendants()
            if e.friendly_class_name() == "Edit" and e.class_name() == "TRzEditNumber"
        ]

        # Verifica os textos capturados
        for i, campo in enumerate(numeros):
            print(f"[{i}] -> {campo.window_text()}")

        # Atribui valores individuais
        diferenca = numeros[0].window_text()
        total_debito = numeros[1].window_text()
        total_credito = numeros[2].window_text()
        lotesMarcados = False
        print("Diferença:", diferenca)
        print("Total Débito:", total_debito)
        print("Total Crédito:", total_credito)

        await worker_sleep(10)
        clicou = False

        if total_credito != total_debito or diferenca != "0,00":
            try:
                lotesMarcados = True
                # Tenta encontrar exatamente o checkbox com o texto "Lotes Consistentes"
                checkbox = dlg.child_window(
                    title="Lotes Consistentes", class_name="TCheckBox"
                )

                if (
                    checkbox.exists()
                    and checkbox.is_enabled()
                    and checkbox.is_visible()
                ):
                    checkbox.click_input()
                    print("Checkbox 'Lotes Consistentes' clicado com sucesso.")

                await worker_sleep(10)
                # Captura todos os campos do tipo TRzEditNumber apos atualizacao de tela..
                numeros = [
                    e
                    for e in dlg.descendants()
                    if e.friendly_class_name() == "Edit"
                    and e.class_name() == "TRzEditNumber"
                ]

                # Verifica os textos capturados
                for i, campo in enumerate(numeros):
                    print(f"[{i}] -> {campo.window_text()}")

                # Atribui valores individuais
                diferenca = numeros[0].window_text()
                total_debito = numeros[1].window_text()
                total_credito = numeros[2].window_text()

                print("Diferença:", diferenca)
                print("Total Débito:", total_debito)
                print("Total Crédito:", total_credito)

                if diferenca > "0,00":
                    clicou = True
                    main_window_capture = main_window.capture_as_image()
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno="Integração não realizada, pois os valores de crédito e débito divergem mesmo após clicar em lotes consistentes.",
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
                    )
            except Exception as e:
                print("Erro inesperado:", e)
        if not clicou:
            print("Clicar no botão integrar")
        # Clicar em integrar
        try:
            # Conecta ao app pela classe principal
            app = Application(backend="win32").connect(
                class_name="TFrmIntegrador", found_index=0
            )
            main_window = app["TFrmIntegrador"]

            # Buscar botão pelo texto (mesmo com acento)
            botao_integrar = main_window.child_window(
                title="Integrar Lançamentos", class_name="TBitBtn"
            ).wrapper_object()

            botao_integrar.click_input()

            await worker_sleep(5)

            assets_int_cont = "assets\\integracao_contabil\\"
            err_dict = {
                assets_int_cont
                + "erro_duplicidade.png": "Integração não realizada. Erro de Duplicidade localizado enquanto finalizava a integração, contate o suporte do Emsys.",
                assets_int_cont
                + "conta_indefinida_error.png": "Integração não realizada. Conta contábil indefinida no sistema.",
                assets_int_cont
                + "lote_sem_complemento_error.png": "Integração não realizada. Lote encontrado sem complemento obrigatório.",
                assets_int_cont
                + "diferenca_cred_deb.png": "Integração não realizada. Existem diferença em lotes consistentes, por favor verificar.",
            }
            # Aguardar finalizar
            while True:
                try:
                    app = Application(backend="win32").connect(
                        class_name="TMsgBox", found_index=0
                    )
                    msg_box = app["TMsgBox"]

                    # Antes de qualquer coisa, verifica por imagem de erro
                    for img_path, mensagem in err_dict.items():
                        try:
                            err = pyautogui.locateOnScreen(img_path, confidence=0.86)
                        except:
                            continue
                        if err:
                            console.print(f"[red]Erro encontrado:[/red] {mensagem}")

                            # Clique no botão OK da mensagem de erro antes de retornar
                            msg_box.child_window(
                                class_name="TBitBtn", found_index=0
                            ).click_input()

                            return RpaRetornoProcessoDTO(
                                sucesso=False,
                                retorno=mensagem,
                                status=RpaHistoricoStatusEnum.Falha,
                                tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
                            )

                    await worker_sleep(1)

                    try:
                        app = Application(backend="win32").connect(
                            class_name="TMsgBox", found_index=0
                        )
                        main_window = app["TMsgBox"]
                        main_window.child_window(
                            class_name="TBitBtn", found_index=0
                        ).click_input()
                        break
                    except ElementNotFoundError:
                        console.print(
                            "[yellow]Janela TMsgBox ainda não visível.[/yellow]"
                        )
                        break

                except ElementNotFoundError:
                    console.print("[yellow]Janela TMsgBox ainda não visível.[/yellow]")
                except Exception as e:
                    print(f"Erro inesperado ao verificar janela de confirmação: {e}")
                    break

                await worker_sleep(1)

                try:
                    app = Application(backend="win32").connect(
                        class_name="TMsgBox", found_index=0
                    )
                    main_window = app["TMsgBox"]
                    main_window.child_window(
                        class_name="TBitBtn", found_index=0
                    ).click_input()
                    time.sleep(5)
                    break
                except ElementNotFoundError:
                    console.print("[yellow]Janela TMsgBox ainda não visível.[/yellow]")

            if lotesMarcados:
                if (
                    checkbox.exists()
                    and checkbox.is_enabled()
                    and checkbox.is_visible()
                ):
                    checkbox.click_input()
                    print("Checkbox 'Lotes Consistentes' desmarcado com sucesso.")
                time.sleep(5)
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Integração realizada, porém, existem LOTES INCONSISTENTES.",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)],
                )
            else:
                return RpaRetornoProcessoDTO(
                    sucesso=True,
                    retorno=f"Sucesso ao executar processo de integracao contabil",
                    status=RpaHistoricoStatusEnum.Sucesso,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                )
        except Exception as erro:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro durante o processo integração contabil, erro : {erro}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )

    except Exception as erro:
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=f"Erro durante o processo integração contabil, erro : {erro}",
            status=RpaHistoricoStatusEnum.Falha,
            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
        )



import asyncio
import getpass
import warnings
import os
import re
import io
import json
from pathlib import Path
import uuid
import pandas as pd
import pyautogui
import pytesseract
from datetime import datetime, timedelta
from pywinauto.application import Application
from pypdf import PdfReader
from PIL import Image, ImageEnhance
from pywinauto.keyboard import send_keys
from pywinauto.mouse import double_click
import win32clipboard
from pywinauto_recorder.player import set_combobox
from rich.console import Console
from worker_automate_hub.api.datalake_service import send_file_to_datalake
from worker_automate_hub.api.client import (
    get_config_by_name,
    get_status_nf_emsys,
    send_file,
)
from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
    RpaTagDTO,
    RpaTagEnum,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)
from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.utils_nfe_entrada import EMSys
from worker_automate_hub.utils.util import (
    e_ultimo_dia_util,
    delete_xml,
    find_nop_divergence,
    find_warning_nop_divergence,
    ocr_warnings,
    ocr_by_class,
    nf_busca_nf_saida,
    pessoas_ativa_cliente_fornecedor,
    nf_devolucao_liquidar_cupom,
    gerenciador_nf_header,
    cadastro_pre_venda_header,
    incluir_registro,
    is_window_open,
    is_window_open_by_class,
    kill_all_emsys,
    login_emsys,
    ocr_title,
    select_documento_type,
    set_variable,
    type_text_into_field,
    worker_sleep,
    post_partner,
    get_text_display_window,
)

pyautogui.PAUSE = 0.5
pyautogui.FAILSAFE = False
console = Console()
emsys = EMSys()


async def open_contabil_processes():
    try:
        console.print("Abrindo EMSys Contabil...")
        os.startfile("C:\\Users\\automatehub\\Desktop\\Contabil\\contabil1.lnk")
        await worker_sleep(3)
        os.startfile("C:\\Users\\automatehub\\Desktop\\Contabil\\contabil2.lnk")
        await worker_sleep(30)
        os.startfile("C:\\Users\\automatehub\\Desktop\\Contabil\\contabil3.lnk")
        await worker_sleep(20)
        pyautogui.hotkey("win", "d")
        await worker_sleep(4)
        os.startfile("C:\\Users\\automatehub\\Desktop\\Contabil\\contabil4.lnk")
        await worker_sleep(2)
    except Exception as error:
        console.print(f"Error: {error}")


async def extracao_fechamento_contabil(
    task: RpaProcessoEntradaDTO,
) -> RpaRetornoProcessoDTO:
    """
    Processo que relazia entrada de notas no ERP EMSys(Linx).

    """
    try:
        # DEFINIR CONSTANTE DEFAULT PARA O ASSETS
        ASSETS_PATH = "assets"
        # Get config from BOF
        config = await get_config_by_name("login_emsys")

        console.print(task)

        # Seta config entrada na var nota para melhor entendimento
        periodo_inicial = task.configEntrada["periodoInicial"]
        periodo_final = task.configEntrada["periodoFinal"]

        # Definindo Variaveis com Escopo global
        data_hoje = datetime.today().strftime("%d/%m/%Y")
        multiplicador_timeout = int(float(task.sistemas[0].timeout))
        set_variable("timeout_multiplicador", multiplicador_timeout)

        # VERIFICANDO ENTRADA
        historico_id = task.historico_id
        if historico_id:
            console.print("Historico ID recuperado com sucesso...\n")
        else:
            console.print(
                "Não foi possivel recuperar o histórico do ID, não sendo possivel enviar os arquivo gerados como retorno...\n"
            )
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno="Não foi possivel recuperar o histórico do ID, não sendo possivel enviar os arquivo gerados como retorno",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )

        try:
            multiplicador_timeout = int(float(task.sistemas[0].timeout))
            set_variable("timeout_multiplicador", multiplicador_timeout)
            await kill_all_emsys()
            await open_contabil_processes()
            config = await get_config_by_name("login_emsys_contabil")

            app = None
            max_attempts = 30
            console.print("Tentando encontrar janela de login...")
            for attempt in range(max_attempts):
                try:
                    app = Application(backend="win32").connect(
                        title="Selecione o Usuário para autenticação"
                    )
                    console.print("Janela encontrada!")
                    break
                except:
                    console.print("Janela ainda nao encontrada...")
                    await worker_sleep(2)
            if not app:
                console.print("Nao foi possivel encontrar a janela de login...")
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno="Erro durante tentativa localizacao de janelas...",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                )
            await emsys.verify_warning_and_error("Erro", "&Ok")
            await worker_sleep(10)
            pyautogui.click(x=1021, y=127)
            console.print("Logando...")
            await emsys.verify_warning_and_error("Erro", "&Ok")
            pyautogui.write(config.conConfiguracao.get("user"))
            pyautogui.press("enter")

            await worker_sleep(4)
            pyautogui.write(config.conConfiguracao.get("pass"))
            pyautogui.press("enter")
            await worker_sleep(10)

            main_window = None
            for attempt in range(max_attempts):
                main_window = Application().connect(title="EMSys [Contabil]")
                main_window = main_window.top_window()
                if main_window.exists():
                    console.print("Janela encontrada!")
                    break
                console.print("Janela ainda nao encontrada...")
                await worker_sleep(1)

            if not main_window:
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno="Erro durante tentativa localizacao de janelas....",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                )

            # Adicionando foco
            try:
                main_window.set_focus()
                console.print(f"Ativando janela: {main_window}")
            except Exception as error:
                console.print(f"Erro ao setar foco na janela: {main_window}")

            await worker_sleep(5)

            # Clicar em BAL
            console.print("Clicanco em BAL")
            pyautogui.click(x=450, y=94)

            await worker_sleep(5)

            ##### Janela Balancete #####

            console.log("Buscando janela Balancete")
            app = Application(backend="win32").connect(title="Balancete", found_index=0)
            main_window = app["Balancete"]

            # Inserir período inicial
            console.print("Inserindo Data Inicial")
            periodo_inicial_temp = periodo_inicial.replace("/", "")
            input_data_inicial = main_window.child_window(
                class_name="TRzEditDate", found_index=1
            ).click()

            await worker_sleep(1)

            type_text_into_field(
                text=periodo_inicial_temp,
                field=input_data_inicial,
                empty_before=False,
                chars_to_empty="0",
            )

            # Inserir período final
            console.print("Inserindo Data Final")
            periodo_final_temp = periodo_final.replace("/", "")
            input_data_final = main_window.child_window(
                class_name="TRzEditDate", found_index=0
            ).click()

            await worker_sleep(1)

            type_text_into_field(
                text=periodo_final_temp,
                field=input_data_final,
                empty_before=False,
                chars_to_empty="0",
            )
            console.print("Selecionar complemento")
            # Seleciona complemento
            select_complemento = main_window.child_window(
                class_name="TRzComboBox", found_index=0
            )
            select_complemento.select("Complemento")

            console.print("Marcar filiais")
            # Selecionar filiais
            chk_selecionar_filiais = main_window.child_window(
                title="Selecionar Filiais", class_name="TRzCheckBox"
            )
            chk_selecionar_filiais.click_input()

            await worker_sleep(1)

            console.print("Clicar em gerar relatório")
            # Clica no botão "Gerar Relatório"
            main_window.child_window(
                title="Gerar Relatório", class_name="TBitBtn"
            ).click()

            ###### Janela seleção de Empresas #####

            console.print("Clicar em selecionar todas as empresas")
            # Selecionar todas empresas
            pyautogui.click(x=673, y=613)

            await worker_sleep(2)

            # Clicar em ok
            app = Application(backend="win32").connect(
                class_name="TFrmSelecaoEmpresasRel", found_index=0
            )
            main_window = app["TFrmSelecaoEmpresasRel"]
            botao_ok = main_window.child_window(class_name="TBitBtn", found_index=1)
            botao_ok.click_input()
            await worker_sleep(1)

            ##### Janela de pré visualização do relatório
            while True:
                console.print("Aguardando janela de pré visualização")
                try:
                    app = Application(backend="win32").connect(
                        class_name="TFrmPreviewRelatorio", found_index=0
                    )
                    main_window = app["TFrmPreviewRelatorio"]
                    console.print("Janela encontrada!")
                    break
                except Exception:
                    await worker_sleep(10)

            await worker_sleep(5)

            console.print("Clicar no botão salvar")
            # Clicar  em Salvar
            pyautogui.click(x=50, y=35)

            await worker_sleep(3)

            ##### Janela Configuração para salvar arquivo #####
            app = Application(backend="win32").connect(
                class_name="TFrmRelatorioFormato"
            )

            # Pegando a janela principal
            main_window = app["TFrmRelatorioFormato"]

            console.print("Selecionar item Excel")
            # Selecionando o item "Excel"
            select_pdf = main_window.child_window(class_name="TComboBox")
            select_pdf.select("Excel")

            await worker_sleep(1)

            # Clicar no botão ok
            botao_ok = main_window.child_window(class_name="TBitBtn", found_index=1)
            botao_ok.click_input()

            await worker_sleep(5)

            ##### Janela Salvar para arquivo #####

            app = Application(backend="win32").connect(
                title="Salvar para arquivo", found_index=0
            )
            main_window = app["Salvar para arquivo"]
            date_now = datetime.now().strftime("%Y%m%d%H%M%S")
            data_inicial_arquivo = periodo_inicial.replace("/", "")
            data_final_arquivo = periodo_final.replace("/", "")

            # Caminho completo para Downloads
            nome_arquivo = f"C:\\Users\\{getpass.getuser()}\\Downloads\\balancete_{data_inicial_arquivo}_{data_final_arquivo}_{date_now}.XLS"

            console.print(f"Salvar arquivo: {nome_arquivo}")

            # Inserir nome do arquivo
            input_nome = main_window.child_window(class_name="Edit", found_index=0)
            type_text_into_field(
                nome_arquivo, input_nome, empty_before=False, chars_to_empty="0"
            )

            await worker_sleep(2)

            # Clicar em salvar
            botao_salvar = main_window.child_window(class_name="Button", found_index=0)
            botao_salvar.click_input()

            await worker_sleep(2)

            ##### Janela Print #####

            app = Application(backend="win32").connect(title="Print")
            main_window = app.window(title="Print")

            # localizar o ComboBox da impressora
            printer_combo = main_window.child_window(
                title_re="Microsoft.*", class_name="TComboBox"
            ).wrapper_object()

            # Selecionar o Microsoft Print to PDF
            printer_combo.select("Microsoft Print to PDF")

            await worker_sleep(3)

            # Seleciona diretamente o ComboBox correto
            select_xls = main_window.descendants(class_name="TComboBox")[2]  # índice 2
            select_xls.select("Excel File")

            # Clica no botão 'OK'
            main_window.child_window(title="OK", class_name="TButton").click()

            await worker_sleep(5)

            username = getpass.getuser()

            await worker_sleep(3)

            console.print("Criar arquivo JSON")

            arquivo_path = Path(nome_arquivo)
            # Altera a extensão para .XLS maiúsculo (caso o EMSys exporte assim)
            caminho_arquivo = arquivo_path.with_suffix(".XLS")
            # Altera a extensão final para .xls minúsculo
            caminho_ajustado = caminho_arquivo.with_suffix(".xls")
            nome_com_extensao = caminho_ajustado.name
            print(nome_com_extensao)
            # Renomeia o arquivo
            os.rename(caminho_arquivo, caminho_ajustado)

            console.print(f"Arquivo renomeado para: {caminho_ajustado}")
            # Caminho do arquivo
            arquivo = caminho_ajustado

            # Lê o Excel (linhas e colunas corretas)
            df = pd.read_excel(arquivo, skiprows=10, usecols=[1, 2, 10], header=None)

            # Renomeia as colunas
            df.columns = ["contaContabil", "contaMovimento", "valor"]

            # Filtra apenas as linhas com 'contaContabil' numérico
            df = df[pd.to_numeric(df["contaContabil"], errors="coerce").notnull()]

            # Converte 'contaContabil' para inteiro como string (ex: 123 -> "123")
            df["contaContabil"] = df["contaContabil"].astype(int).astype(str)

            # Converte a coluna 'valor' para float, ignorando erros
            df["valor"] = pd.to_numeric(df["valor"], errors="coerce").fillna(0)

            # Formata o valor como moeda brasileira
            df["valor"] = df["valor"].apply(
                lambda x: f"{x:,.2f}".replace(".", "X")
                .replace(",", ".")
                .replace("X", ",")
            )

            # Cria o JSON no formato final
            dados_json = {
                "balancete": {
                    "periodoInicial": periodo_inicial,
                    "periodoFinal": periodo_final,
                    "dataHora": datetime.now().strftime("%Y-%m-%d %H:%M"),
                },
                "saldos": df.to_dict(orient="records"),
            }

            # Salva o JSON

            # Caminho completo do arquivo
            nome_sem_extensao = caminho_ajustado.stem
            full_path = f"C:\\Users\\{username}\\Downloads\\{nome_sem_extensao}.json"
            filename = os.path.basename(full_path)
            with open(full_path, "w", encoding="utf-8") as f:
                json.dump(dados_json, f, ensure_ascii=False, indent=2)

            # Exibe o JSON formatado
            console.print(json.dumps(dados_json, ensure_ascii=False, indent=2))

            await worker_sleep(3)
            sended_to_datalake = False
            console.print("Enviar arquivo para o Datalake")
            # Envia o JSON para o datalake
            directory = "balancete_contabil/raw"

            with open(full_path, "rb") as file:
                file_bytes = io.BytesIO(file.read())
            try:
                console.print("Enviando Json para data lake")
                await send_file_to_datalake(directory, file_bytes, filename, "json")
                sended_to_datalake = True
            except Exception as e:
                console.print(f"Erro ao enviar o arquivo: {e}", style="bold red")
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Erro ao enviar o arquivo: {e}",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                )
            sended_to_bof = False
            # lê o arquivo
            with open(f"{caminho_ajustado}", "rb") as file:
                file_bytes = io.BytesIO(file.read())

            console.print("Enviar Excel para o BOF")
            # Envia o arquivo para o BOF
            try:
                await send_file(
                    historico_id,
                    nome_com_extensao,
                    "xls",
                    file_bytes,
                    file_extension="xls",
                )
                console.print("Removendo arquivo XLS da pasta downloads")
                os.remove(f"{caminho_ajustado}")
                console.print("Removendo arquivo JSON da pasta downloads")
                os.remove(full_path)
                sended_to_bof = True
            except Exception as e:
                result = f"Arquivo Balancete contábil gerado com sucesso, porém gerou erro ao realizar o envio para o backoffice {e} - Arquivo ainda salvo na dispositivo utilizado no diretório {caminho_arquivo} !"
                console.print(result, style="bold red")
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=result,
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
                )

            if sended_to_datalake and sended_to_bof:
                return RpaRetornoProcessoDTO(
                    sucesso=True,
                    retorno=json.dumps(dados_json),
                    status=RpaHistoricoStatusEnum.Sucesso,
                )
        except Exception as erro:
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=f"Erro durante o processo integração contabil, erro : {erro}",
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
            )

    except Exception as ex:
        retorno = f"Erro Processo Fechamento Balancete: {str(ex)}"
        logger.error(retorno)
        console.print(retorno, style="bold red")
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=retorno,
            status=RpaHistoricoStatusEnum.Falha,
            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)],
        )



import asyncio

from pydantic import ValidationError
from rich.console import Console
from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
    RpaTagDTO,
    RpaTagEnum,
)
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import (
    RpaProcessoEntradaDTO,
)

console = Console()


async def playground(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    try:
        console.print(
            f"\nProcesso de teste iniciado: {task.nomProcesso}\n", style="green"
        )
        max_attempts = 15
        for numero in range(1, max_attempts):
            console.print(f"Etapa [{numero}] de {max_attempts}", style="green")
            await asyncio.sleep(1)
        console.print(f"Processo de teste finalizado.", style="green")
        return RpaRetornoProcessoDTO(
            sucesso=True,
            retorno="Processo de teste executado com sucesso",
            status=RpaHistoricoStatusEnum.Sucesso,
        )
    except ValidationError as exc:
        console.print(
            f"Erro de validação ao executar o processo de teste: {repr(exc.errors()[0]['type'])}",
            style="bold red",
        )
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=f"Erro de validação ao executar o processo de teste: {repr(exc.errors()[0]['type'])}",
            status=RpaHistoricoStatusEnum.Falha,
            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
        )

    except Exception as e:
        console.print(f"Erro ao executar o processo de teste: {e}", style="bold red")
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=f"Erro ao executar o processo de teste: {e}",
            status=RpaHistoricoStatusEnum.Falha,
            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
        )



import asyncio
import io
import os
import re
import shutil
import warnings
from datetime import datetime, timedelta
from decimal import ROUND_HALF_UP, Decimal

import pyperclip
import pyautogui
from pywinauto.application import Application
from rich.console import Console
from dateutil.relativedelta import relativedelta

from worker_automate_hub.models.dto.rpa_historico_request_dto import RpaHistoricoStatusEnum, RpaRetornoProcessoDTO, RpaTagDTO, RpaTagEnum
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import RpaProcessoEntradaDTO
from worker_automate_hub.utils.get_creds_gworkspace import GetCredsGworkspace
from googleapiclient.discovery import build
from worker_automate_hub.utils.logger import logger
from pywinauto_recorder import set_combobox
from worker_automate_hub.api.client import get_config_by_name, get_valor_remessa_cobranca, send_file, sync_get_config_by_name
from worker_automate_hub.utils.util import (
    create_temp_folder,
    delete_folder,
    kill_all_emsys,
    login_emsys,
    save_pdf_emsys, 
    type_text_into_field,
    worker_sleep,
    set_variable,
    )

pyautogui.PAUSE = 0.5
ASSETS_BASE_PATH = 'assets/fidc/'
PDF_SUFFIX = "_PDF.pdf"
console = Console()



async def remessa_cobranca_cnab240(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    '''
       Processo FIDC - Remessa de Cobrança CNAB240
    '''
    try:
        #Setando tempo de timeout
        multiplicador_timeout = int(float(task.sistemas[0].timeout))
        set_variable("timeout_multiplicador", multiplicador_timeout)
        
        #Pegando nome do usuario
        nome_usuario = os.environ.get('USERNAME') or os.environ.get('USER')
        nome_pasta = f"{nome_usuario}_arquivos"

        #Delete temp folder
        await delete_folder(nome_pasta)
        #Cria Pasta temporaria
        temp_folder = await create_temp_folder()
        temp_folder = temp_folder +'\\'
        #Pega Config para logar no Emsys
        config = await get_config_by_name("login_emsys")
        folders_paths = await get_config_by_name("Folders_Fidc")

        #Abre um novo emsys
        await kill_all_emsys()
        app = Application(backend='win32').start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings("ignore", category=UserWarning, message="32-bit application should be automated using 32-bit Python")
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config.conConfiguracao, app, task)

        if return_login.sucesso == True:
            type_text_into_field('Remessa de Cobranca', app['TFrmMenuPrincipal']['Edit'], True, '50')
            pyautogui.press('enter')
            await worker_sleep(1)
            pyautogui.press('enter')
            console.print(f"\nPesquisa: 'Retorno de Cobranca' realizada com sucesso", style="bold green")
        else:
            logger.info(f"\nError Message: {return_login.retorno}")
            console.print(f"\nError Message: {return_login.retorno}", style="bold red")
            return return_login
        
        await worker_sleep(10)
        
        # Identificando jenela principal
        app = Application().connect(title="Gera Arquivo Cobranca", backend="uia")
        main_window_arquivo_cobranca = app["Gera Arquivo Cobranca"]
        main_window_arquivo_cobranca.set_focus()

        # Digitando Cobrança
        cobranca = main_window_arquivo_cobranca.child_window(class_name="TDBIEditCode", found_index=2)
        console.print("Selecionando Cobrança", style='bold green')
        cobranca.type_keys("4")
        pyautogui.hotkey("tab") 
        await worker_sleep(5)
        pyautogui.press("down", presses=3, interval=0.5)
        await worker_sleep(2)
        pyautogui.hotkey("enter")

        await worker_sleep(10)
        
        # Identificando jenela principal
        app = Application().connect(title="Gera Arquivo Cobranca", backend="uia")
        main_window_arquivo_cobranca = app["Gera Arquivo Cobranca"]

        # Selecionando caminho do arquivo
        field_arquivo = main_window_arquivo_cobranca.child_window(class_name="TDBIEditString", found_index=0)
        text_field_arquivo = field_arquivo.window_text()
        new_text_field_arquivo = str(re.search(r'REM(.*)\.txt', text_field_arquivo).group(1))
        new_text_field_arquivo = 'R00102#####.001'.replace('#####', str(new_text_field_arquivo).zfill(5))
        field_arquivo.set_focus()
        
        field_arquivo.double_click_input()
        field_arquivo.set_edit_text("")
        # field_arquivo.type_keys(folders_paths.conConfiguracao['remessa_cobranca_path'] + new_text_field_arquivo, with_spaces=True)
        field_arquivo.type_keys(temp_folder + new_text_field_arquivo, with_spaces=True)

        await worker_sleep(2)   
        #Seleciona Banco
        pyautogui.click(810, 397)
        pyautogui.press("down", presses=2)

        pyautogui.hotkey("enter")
        
        # Data atual
        data_atual = datetime.now()

        # Data(8 dias atrás)
        start_date = data_atual - timedelta(days=8)
        # Data(1 dia atrás)
        end_date = data_atual - timedelta(days=1)


        #Data de emissão
        pyautogui.click(700, 482)
        pyautogui.write(start_date.strftime("%d%m%Y"))
        pyautogui.click(780, 485)
        pyautogui.write(end_date.strftime("%d%m%Y"))

        #Data Vencimento
        pyautogui.click(900, 485)
        pyautogui.write(datetime.now().strftime("%d%m%Y"))
        # pyautogui.write(start_date)
        pyautogui.click(1000, 485)
        pyautogui.write((datetime.now() + relativedelta(months=6)).strftime("%d%m%Y"))
        # pyautogui.write(end_date)

        filtro = main_window_arquivo_cobranca.child_window(class_name="TGroupBox", found_index=2)
        faturados_negociados = filtro.child_window(title="Faturados/Negociados", class_name="TRadioButton")
        faturados_negociados.click()
        somente_nosso_numero = main_window_arquivo_cobranca.child_window(title="Somente Títulos com Nosso Número",class_name="TCheckBox", found_index=0)
        somente_nosso_numero.click()

        # Clica Pesquisar Titulos
        pyautogui.click(1160, 548)

        await worker_sleep(10)

        #Clicando em sim para titulos
        pyautogui.click(920, 560)

        await worker_sleep(20)

        #Selecionando todas empresas na tela  Seleção de Empresa
        pyautogui.click(720, 620)

        #clcica em OK
        pyautogui.click(1100, 660)

        await worker_sleep(120)

        try:
            app = Application().connect(title="Informação", backend="uia")
            main_window_arquivo_cobranca = app["Informação"]
            if main_window_arquivo_cobranca.exists():
                return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno="Nenhum titulo encontrado para a remessa de cobrança!",
            status=RpaHistoricoStatusEnum.Sucesso,
        )
        except Exception as ex:
            ...
        # Seleciona todos titulos
        pyautogui.click(1275, 600)
        # pyautogui.click(636, 591)
        
        await worker_sleep(10)

        # Pegando Total do Emsys
        app = Application().connect(title="Gera Arquivo Cobranca", backend="uia")
        main_window_arquivo_cobranca = app["Gera Arquivo Cobranca"]
        main_window_arquivo_cobranca.set_focus()
        field_total_emsys = main_window_arquivo_cobranca.child_window(class_name="TDBIEditNumber", found_index=3).window_text()
        #Pegando total do banco
        total_db = await get_valor_remessa_cobranca(data_atual.strftime("%Y-%m-%d"))
        #Compara valores
        if total_db == Decimal(field_total_emsys.replace('.', '').replace(',', '.')):
            #Clica gerar cobrança
            await worker_sleep(5)
            pyautogui.click(1135, 789)
        else:
            log_msg = "Valores divergem! \nValor no EmSys: " + str(field_total_emsys.replace('.', '').replace(',', '.')) + " \nValor dos titulos: " + str(total_db)
            return RpaRetornoProcessoDTO(
                sucesso=False, 
                retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, 
                tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)])    

        await worker_sleep(10)

        # Confirma geração com sucesso
        max_try = 10
        current_try = 0
        while current_try <= max_try:
            try:
                app = Application().connect(title="Information", class_name="TMessageForm")
                window_cobranca = app["Information"]
                if window_cobranca.exists():
                    #Click OK
                    window_cobranca.set_focus()
                    await worker_sleep(1)
                    pyautogui.click(958, 560)
                    break
            except Exception as ex:
                log_msg = f"Erro ao encontrar janela de confirmação de cobrança: {str(ex)}"
                console.print(log_msg, style="bold red")
                
            current_try += 1
            await worker_sleep(20)
            
        if current_try >= max_try:
            return RpaRetornoProcessoDTO(
                sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])    

        # # Substituindo arquivo original e fazedno upload (feito logo apos gerar para  garaintir que o arquivo não se perca)
        # origem = folders_paths.conConfiguracao['remessa_cobranca_path'] + new_text_field_arquivo
        origem = temp_folder + new_text_field_arquivo
        
        # Abrindo o arquivo original para leitura
        file = open(origem, 'r')
        file_text = file.read()
        file.close()
        # Realizando as substituições
        file_text = file_text.replace(
        '074737350001813576383             0316820000000058114 SIM REDE DE POSTOS LTDA',
        '45931917000148003576383001417019  0316820000000058114 ARGENTA FUNDO DE INVEST').replace(
        '20074737350001813576383             0316820000000058114 SIM REDE DE POSTOS LTDA',
        '2045931917000148003576383001417019  0316820000000058114 ARGENTA FUNDO DE INVEST')
        # Sobrescrevendo o arquivo original com o conteúdo alterado
        file = open(origem, 'w')
        file.write(file_text)
        file.close()

        console.print(f"Substituições realizadas com sucesso no arquivo original: {origem}")
        try:
            with open(origem, 'rb') as file:
                file_bytes = io.BytesIO(file.read())
            # Enviando o arquivo 001 para BOF
            await send_file(task.historico_id, new_text_field_arquivo, "001", file_bytes, file_extension="001")
        except Exception as e:
            log_msg=(f"Erro ao enviar o arquivo 001: {e}")
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(
            sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])


        # Clica em 'Yes' imprimir listagem
        await worker_sleep(10)
        try:
            app = Application().connect(title="Confirm", class_name="TMessageForm")
            window_listagem_alfabetica = app["Confirm"]
            if window_listagem_alfabetica.exists():
                window_listagem_alfabetica.set_focus()
                yes_btn = window_listagem_alfabetica.child_window(title="&Yes", class_name="TButton")
                yes_btn.click()
        except Exception as ex:
            log_msg = f"Erro ao encontrar janela de impressão de listagem: {str(ex)}"
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(
            sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])
        
        #Clica em "Yes" ordem alfabética
        await worker_sleep(10)
        try:
            app = Application().connect(title="Confirm", class_name="TMessageForm")
            window_listagem_alfabetica = app["Confirm"]
            if window_listagem_alfabetica.exists():
                window_listagem_alfabetica.set_focus()
                yes_btn = window_listagem_alfabetica.child_window(title="&Yes", class_name="TButton")    
                yes_btn.click()  
        except Exception as ex:
            log_msg = f"Erro ao encontrar janela de impressão de listagem em ordem alfabética: {str(ex)}"
            console.print(log_msg, style="bold red")            
            return RpaRetornoProcessoDTO(
            sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])

        await worker_sleep(10)
        
        # await save_pdf_emsys(folders_paths.conConfiguracao['remessa_cobranca_path'] + new_text_field_arquivo +" (PDF)")
        await save_pdf_emsys(temp_folder + new_text_field_arquivo + PDF_SUFFIX)
        
        await worker_sleep(5) 

        #Clica para não imprimir os boletos
        try:
            app = Application().connect(title="Confirm", class_name="TMessageForm")
            window_listagem_alfabetica = app["Confirm"]
            if window_listagem_alfabetica.exists():
                window_listagem_alfabetica.set_focus()
                pyautogui.click(998, 562)
        except Exception as ex:
            log_msg = f"Erro ao encontrar janela 'deseja imprimir os boletos?': {str(ex)}"
            console.print(log_msg, style="bold red")            
            return RpaRetornoProcessoDTO(
            sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])
                
        try:
            # Enviando arquivo PDF para BOF
            with open(temp_folder + new_text_field_arquivo + PDF_SUFFIX, 'rb') as file:
                file_bytes_pdf = io.BytesIO(file.read())
            await send_file(task.historico_id, new_text_field_arquivo, "pdf", file_bytes_pdf, file_extension="pdf")
        except Exception as e:
            log_msg=(f"Erro ao enviar o arquivo PDF: {e}")
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(
            sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])
        
        try:
            #Mover para pasta correta
            shutil.move(origem, folders_paths.conConfiguracao["remessa_cobranca_path_prod"])
        except Exception as e:
            log_msg=(f"Erro ao mover o arquivo para pasta de remessa: {e}")
            console.print(log_msg, style="bold red")
            return RpaRetornoProcessoDTO(
            sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])
    
        #Delete temp folder
        await delete_folder(nome_pasta)
        return RpaRetornoProcessoDTO(
            sucesso=True,
            retorno="Processo Remessa de Cobranca CNAB240 concluido com sucesso",
            status=RpaHistoricoStatusEnum.Sucesso,
        )

    except Exception as ex:
        log_msg = f"Erro Processo Remessa de Cobranca CNAB240: {str(ex)}"
        console.print(log_msg, style="bold red")
        return RpaRetornoProcessoDTO(
        sucesso=False, retorno=log_msg, status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)])
        


import asyncio
import io
import os
import json
import shutil
from datetime import datetime
import requests
from pydantic import BaseModel
from typing import Optional
from rich.console import Console
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from webdriver_manager.chrome import ChromeDriverManager
from selenium.webdriver.common.by import By
from selenium.webdriver.common.action_chains import ActionChains
from selenium.webdriver.common.keys import Keys
from worker_automate_hub.api.client import get_config_by_name, send_file
from worker_automate_hub.api.datalake_service import send_file_to_datalake
from worker_automate_hub.models.dto.rpa_historico_request_dto import (
    RpaHistoricoStatusEnum,
    RpaRetornoProcessoDTO,
    RpaTagDTO,
    RpaTagEnum
)
from worker_automate_hub.models.dto.rpa_sap_dto import RpaProcessoSapDTO
from worker_automate_hub.utils.util import worker_sleep

console = Console()

DOWNLOADS_PATH = os.path.join(os.path.expanduser("~"), "Downloads")

console.print(f"Downloads dir: {DOWNLOADS_PATH}")

class ConfigEntradaSAP(BaseModel):
    user: str
    password: str
    empresa: str
    unique_id: Optional[str] = "default"

    def get(self, key, default=None):
        return getattr(self, key, default)

class NotasFaturamentoSAP:
    def __init__(self, task: RpaProcessoSapDTO, sap_url, sap_key, sap_token, base_url, directory):
        console.print("Inicializando classe NotasFaturamentoSAP.")
        self.task = task
        self.sap_url = sap_url
        self.sap_key = sap_key
        self.sap_token = sap_token
        self.user = task.configEntrada.get("user")
        self.password = task.configEntrada.get("password")
        self.base_url = base_url
        self.empresa = task.configEntrada.get("empresa")
        self.directory = directory
        self.unique_id = "default"
        self.driver = None

    async def start_sap_process(self):
        console.print("Iniciando o processo SAP.")
        sim_service = Service(ChromeDriverManager().install())
        self.driver = webdriver.Chrome(service=sim_service)
        self.driver.maximize_window()
        console.print("Driver inicializado e janela maximizada.")

        await self.save_process_pid()
        console.print("PID do processo salvo com sucesso.")
        self.driver.get(self.base_url)
        console.print(f"Acessando a URL base: {self.base_url}")
        await worker_sleep(3)

        if not await self.login():
            msg = f"Falha ao realizar login no SAP"
            console.print(msg)
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=msg,
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

        console.print("Login realizado com sucesso.")
        await self.download_files(self.empresa, "FAT")
        if self.driver:
            try:
                self.driver.quit()
                self.driver = None
                console.print("Driver fechado e referência removida.")
            except Exception as e:
                msg = f"Erro ao fechar o driver: {e}"
                console.print(msg)
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=msg,
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )

    async def save_process_pid(self):
        try:
            console.print("Salvando o PID do processo do Chrome.")
            pid = str(self.driver.service.process.pid)
            file_path = f"c:\\tmp\\chrome_pid_{self.unique_id}.txt"
            with open(file_path, "w") as f:
                f.write(pid)
            console.print(f"PID salvo em: {file_path}")
        except Exception as e:
            msg = f"Erro ao salvar PID: {e}"
            console.print(msg)
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=msg,
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

    async def login(self):
        try:
            console.print("Iniciando login no SAP.")
            for _ in range(30):
                inputs = self.driver.find_elements(By.CLASS_NAME, "loginInputField")
                if len(inputs) >= 2:
                    console.print("Campos de entrada de login encontrados.")
                    break
                await worker_sleep(1)

            inputs = self.driver.find_elements(By.CLASS_NAME, "loginInputField")
            inputs[0].send_keys(self.user)
            console.print("Usuário inserido no campo de login.")
            await worker_sleep(2)
            inputs[1].send_keys(self.password)
            console.print("Senha inserida no campo de login.")
            await worker_sleep(1)

            for _ in range(10):
                login_btn = self.driver.find_elements(By.ID, "LOGIN_SUBMIT_BLOCK")
                if login_btn:
                    login_btn[0].click()
                    console.print("Botão de login clicado.")
                    break
                await worker_sleep(1)

            await worker_sleep(3)
            console.print("Login realizado com sucesso.")
            return True
        except Exception as e:
            console.print(f"Erro ao realizar login: {e}")
            return False

    async def download_files(self, company: str, file_type: str):
        console.print(f"Iniciando download de arquivos para empresa {company} e tipo {file_type}.")
        doc_url = self.get_document_url(file_type)
        console.print(f"Acessando a URL do documento: {doc_url}")
        self.driver.get(doc_url)
        await worker_sleep(15)

        if len(self.driver.find_elements(By.ID, "searchFieldInShell-input-inner")) < 1:
            for _ in range(10):
                sf = self.driver.find_elements(By.ID, "sf")
                if sf:
                    sf[0].click()
                    console.print("Elemento 'sf' clicado.")
                    break
                await worker_sleep(1)

        for _ in range(10):
            if len(self.driver.find_elements(By.ID, "searchFieldInShell-input-inner")) > 0:
                console.print("Campo de busca encontrado.")
                break
            await worker_sleep(1)

        self.driver.find_element(By.ID, "searchFieldInShell-input-inner").click()
        console.print("Campo de busca clicado.")
        await worker_sleep(5)

        actions = ActionChains(self.driver)
        console.print("Executando ações para navegar pelos campos.")
        for _ in range(34):
            actions.send_keys(Keys.TAB)
            actions.perform()
            await worker_sleep(0.5)
        actions.send_keys(Keys.ENTER)
        actions.perform()
        await worker_sleep(5)
        actions.send_keys(Keys.ENTER)
        actions.perform()
        await worker_sleep(5)
        actions.send_keys(Keys.ENTER)
        actions.perform()
        await worker_sleep(15)
        console.print("Ações concluídas.")
        if self.driver:
            try:
                self.driver.quit()
                self.driver = None
                console.print("Driver fechado e referência removida.")
            except Exception as e:
                msg = f"Erro ao fechar o driver: {e}"
                console.print(msg)
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=msg,
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )
        await self.rename_file(company, file_type)

    def get_document_url(self, file_type: str) -> str:
        console.print(f"Obtendo URL do documento para o tipo {file_type}.")
        mapping = {
            "FAT": "#BillingDocument-analyzeRevenue?sap-ui-tech-hint=WDA",
            "DET": "#BillingDocument-analyzeBillingDocItemPricing?sap-ui-tech-hint=WDA",
            "PED": "#SalesOrder-analyzeIncoming?sap-ui-tech-hint=WDA",
        }
        suffix = mapping.get(file_type.upper(), "")
        return f"{self.base_url}{suffix}"

    async def rename_file(self, company: str, file_type: str):
        console.print("Arquivos baixados com sucesso")
        console.print("Iniciando renomeação e movimentação do arquivo.")
        try:
            current_path = os.path.join(DOWNLOADS_PATH, "export.xlsx")
            date_now = datetime.now().strftime("%Y%m%d%H%M%S")
            filename = f"{company}_{file_type}_{date_now}.xlsx"
            final_path = os.path.join(DOWNLOADS_PATH, filename)
            
            console.print(f"filename: {filename}")
            
            os.rename(current_path, final_path)
            console.print(f"Arquivo renomeado para {final_path}.")
            
            with open(final_path, 'rb') as file:
                file_bytes = io.BytesIO(file.read())
            
            await worker_sleep(5)

            await self.send_message_to_webhook(f"Arquivo gerado via RPA: {filename}")

            await worker_sleep(2)

            try:
                console.print(f"directory: {self.directory}")
                console.print(f"file: {final_path}")
                send_file_request = await send_file_to_datalake(self.directory, file_bytes, filename, "xlsx")
                console.print(send_file_request)
            except Exception as e:
                console.print(f"Erro ao enviar o arquivo: {e}", style="bold red")
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Erro ao enviar o arquivo: {e}",
                    status=RpaHistoricoStatusEnum.Falha,
                    tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )
            
            await worker_sleep(2)

            if final_path and os.path.exists(final_path):
                try:
                    os.remove(final_path)
                    console.print(f"Arquivo deletado: {final_path}")
                except Exception as e:
                    msg = f"Erro ao deletar o arquivo: {e}"
                    console.print(msg)
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=msg,
                        status=RpaHistoricoStatusEnum.Falha,
                        tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                    )

        except FileNotFoundError as fne:
            msg = f"Não foi possível renomear o arquivo, error: {fne}"
            console.print(msg)
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=msg,
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )
        except Exception as e:
            msg = f"Erro ao renomear/mover arquivo: {e}"
            console.print(msg)
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=msg,
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

    async def send_message_to_webhook(self, message: str):
        console.print("Enviando mensagem ao webhook.")
        await worker_sleep(2)
        try:
            payload = {"text": "📢 " + message}
            webhook_url = f"{self.sap_url}/key={self.sap_key}&token={self.sap_token}"
            requests.post(
                webhook_url,
                data=json.dumps(payload),
                headers={"Content-Type": "application/json"},
                timeout=10
            )
            console.print("Mensagem enviada ao webhook com sucesso.")
            await worker_sleep(2)
        except Exception as e:
            msg = f"Erro ao enviar mensagem ao webhook: {e}"
            console.print(msg)
            return RpaRetornoProcessoDTO(
                sucesso=False,
                retorno=msg,
                status=RpaHistoricoStatusEnum.Falha,
                tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
            )

async def notas_faturamento_sap(task: RpaProcessoSapDTO) -> RpaRetornoProcessoDTO:
    console.print("Iniciando processo de notas de faturamento SAP.")
    try:
        config = await get_config_by_name("SAP_Faturamento")
        console.print(f"config: {config}")
        sap_url = config.conConfiguracao.get("sapUrl")
        sap_key = config.conConfiguracao.get("sapKey")
        sap_token = config.conConfiguracao.get("sapToken")
        base_url = config.conConfiguracao.get("baseUrl")
        directory = config.conConfiguracao.get("directoryBucket")
        console.print(sap_url, sap_key, sap_token, base_url, directory)
        notas_sap = NotasFaturamentoSAP(task, sap_url, sap_key, sap_token, base_url, directory)
        await notas_sap.start_sap_process()
        console.print("Processo de automação SAP finalizado com sucesso.")
        return RpaRetornoProcessoDTO(
            sucesso=True,
            retorno="Processo de automação SAP finalizado com sucesso.",
            status=RpaHistoricoStatusEnum.Sucesso,
        )
    except Exception as ex:
        console.print(f"Erro na automação SAP: {ex}")
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=f"Erro na automação SAP: {ex}",
            status=RpaHistoricoStatusEnum.Falha,
            tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
        )





import asyncio
from worker_automate_hub.api.client import get_config_by_name
from worker_automate_hub.models.dto.rpa_historico_request_dto import RpaHistoricoStatusEnum, RpaRetornoProcessoDTO, RpaTagDTO, RpaTagEnum
from worker_automate_hub.models.dto.rpa_processo_entrada_dto import RpaProcessoEntradaDTO
from pywinauto import Application
import pyautogui
import warnings
from pywinauto.keyboard import send_keys
from worker_automate_hub.utils.util import kill_all_emsys, type_text_into_field, wait_window_close, worker_sleep
from worker_automate_hub.utils.utils_nfe_entrada import EMSys
from rich.console import Console

from worker_automate_hub.utils.logger import logger
from worker_automate_hub.utils.util import (
    login_emsys,
    set_variable
)
from pytesseract import image_to_string
from PIL import ImageFilter, ImageEnhance
pyautogui.PAUSE = 0.5
console = Console()

emsys = EMSys()


def get_text_from_window(window, relative_coords, value=None):
    try:
        screenshot = window.capture_as_image()
        imagem = screenshot.convert("L")
        imagem = imagem.filter(ImageFilter.SHARPEN)
        imagem = ImageEnhance.Contrast(imagem).enhance(2)
        cropped_screenshot = imagem.crop(relative_coords)
        texto = image_to_string(cropped_screenshot, lang="por")
        return (value.upper() in texto.upper()) if value != None else texto.lower()
    except Exception as error:
        print(f"Error: {error}")


async def geracao_aprovacao_pedidos_34(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    filial_origem = "34"
    periodo = "1"
    retorno = await geracao_aprovacao_pedidos_processo(task=task, filial_origem=filial_origem, periodo=periodo)
    return retorno

async def geracao_aprovacao_pedidos_171(task: RpaProcessoEntradaDTO) -> RpaRetornoProcessoDTO:
    filial_origem = "171"
    periodo = "1"
    retorno = await geracao_aprovacao_pedidos_processo(task=task, filial_origem=filial_origem, periodo=periodo)
    return retorno

async def geracao_aprovacao_pedidos_processo(task: RpaProcessoEntradaDTO, filial_origem: str, periodo: str) -> RpaRetornoProcessoDTO:
    try:
        # Get config from BOF
        config = await get_config_by_name("login_emsys")
        console.print(task)
        console.print(config)
        
        filialEmpresaOrigem = filial_origem
        periodo = periodo
        # Fecha a instancia do emsys - caso esteja aberta
        await kill_all_emsys()
        
        app = Application(backend="win32").start("C:\\Rezende\\EMSys3\\EMSys3.exe")
        warnings.filterwarnings(
            "ignore",
            category=UserWarning,
            message="32-bit application should be automated using 32-bit Python",
        )
        console.print("\nEMSys iniciando...", style="bold green")
        return_login = await login_emsys(config.conConfiguracao, app, task, filial_origem=filialEmpresaOrigem)
        main = app.top_window()
        main.set_focus()

        if return_login.sucesso == True:
            console.print("Indo para aprovação de negociação de compra...")
            type_text_into_field(
                "Aprovação de Negociação", app["TFrmMenuPrincipal"]["Edit"], True, "50"
            )
            await worker_sleep(3)
            pyautogui.press("enter", presses=2)
            await worker_sleep(4)
            aprovacao_window = app.top_window()
            try:
                console.print("Preenchendo campo periodo de dias...")
                aprovacao_window.Edit3.set_text(periodo)
            except Exception as erro:
                console.print(erro)
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Erro durante o processo geracao_aprovacao_pedidos, erro : {erro}",
                    status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )
            try:
                console.print("Preenchendo o campo filial...")
                type_text_into_field(
                    filialEmpresaOrigem, aprovacao_window["Edit2"], True, "0"
                )
                await worker_sleep(5)
                send_keys("{ENTER}")
            except Exception as erro:
                console.print(erro)
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Erro durante o processo geracao_aprovacao_pedidos, erro : {erro}",
                    status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                )
            await worker_sleep(5)
            console.print("Marcando campo selecionar empresas...")
            pyautogui.click(x=942, y=311)
            await worker_sleep(5)
            console.print("Clicando em pesquisar...") 
            pyautogui.click(x=1107, y=333)
            await worker_sleep(5)
            console.print("Selecionando todas as filiais...")
            pyautogui.click(x=720, y=615)
            await worker_sleep(5)
            console.print("Clicando em ok para confirmar...")
            pyautogui.click(x=1090, y=657)
            await worker_sleep(5)

            # tela de aguarde
            try:
                console.print("Aguardando janela de aguarde finalizar...")
                await wait_window_close("Aguarde...")
                await worker_sleep(5)
            except Exception as error:
                console.print("Janela de aguarde nao encontrada, tentando continuar")
                await worker_sleep(5)
            console.print("Tela de aguarde finalizada...")
            await worker_sleep(30)
            pyautogui.click(581, 425)
            await worker_sleep(15)
            # existe_pedidos = get_text_from_window(aprovacao_window, (40, 143, 105, 165), value="empresa")
            existe_pedidos = True
            if not existe_pedidos:
                console.print("Nenhum pedido encontrado...")
                return RpaRetornoProcessoDTO(
                    sucesso=False,
                    retorno=f"Erro durante o processo geracao_aprovacao_pedidos, erro : Nao ha pedidos para o periodo",
                    status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Negocio)]
                )
            
            await worker_sleep(30)
            console.print("Clicando para selecionar todos os registros...")
            pyautogui.click(x=577, y=422)
            console.print("Aguardando processar...")
            await worker_sleep(30)
            pyautogui.click(x=1120, y=751)
            console.print("Clicando em aprovar...")
            await worker_sleep(30)
            console.print("clicando em OK em Motivo da aprovação da Negociação de Compra...")
            await emsys.verify_warning_and_error("Motivo da Aprovação da Negociação de Compra", "&Ok")
            await worker_sleep(10)
            console.print("Confirmando compra aprovada...")
            await emsys.verify_warning_and_error("Information", "OK")
            await emsys.verify_warning_and_error("Warning", "OK")
            await worker_sleep(10)
            console.print("Clicando em yes na janela de envio de email...")
            await emsys.verify_warning_and_error("Confirm", "&No")
            try:
                console.print("Aguardando janela de aguarde finalizar...")
                await wait_window_close("Aguarde...")
                await worker_sleep(5)
            except Exception as error:
                console.print("Janela de aguarde nao encontrada, tentando continuar")
                await worker_sleep(5)
            console.print("Tela de aguarde finalizada...")
            await worker_sleep(10)
            console.print("Marcando a primeira empresa da lista...")
            pyautogui.press("space")
            await worker_sleep(5)
            console.print("Clicando em OK...")
            await emsys.verify_warning_and_error("Seleção de Empresas", "&OK")
            await worker_sleep(5)
            console.print("Indo para janela de pré venda...")
            await worker_sleep(5)
            console.print("Fechando janela de aprovacao...")
            pyautogui.click(x=1329, y=277)
            await worker_sleep(5)
            type_text_into_field(
                "Gera Pré Venda com Pedido de Compra", app["TFrmMenuPrincipal"]["Edit"], True, "50"
            )
            await worker_sleep(3)
            pyautogui.press("enter", presses=2)
            await worker_sleep(5)
            pre_venda_window = app.top_window()
            console.print("Clicando em gerar pre venda...")
            pyautogui.click(x=1274, y=695)
            await worker_sleep(20)
            console.print("Confirmando janela...")
            await emsys.verify_warning_and_error("Confirm", "&Yes")
            await worker_sleep(5)#20
            console.print("Selecionando segunda opção da janela Seleção do Tipo de Preço...")
            pyautogui.click(x=852, y=522)
            await worker_sleep(5)
            console.print("Confirmando opção...")
            pyautogui.press("enter")
            pre_venda_window.wait("ready", timeout=1000)
            await worker_sleep(10)
            console.print("Confirmando janela de pré venda gerada...")
            await emsys.verify_warning_and_error("Information", "OK")
            await emsys.verify_warning_and_error("Information", "&OK")
            await worker_sleep(20)
            console.print("Confirmando janela...")
            await emsys.verify_warning_and_error("Confirm", "&Yes")
            pre_venda_window.wait("ready", timeout=1000)
            await worker_sleep(30)
            try:
                pre_venda_window = app.window("Pré Venda")
                pre_venda_window.wait("ready", timeout=1000)
                await worker_sleep(5)
            except:
                pre_venda_window = app.top_window()
                await worker_sleep(30)
            await worker_sleep(5)
            console.print("Clicando em confirmar pre-venda...")
            pyautogui.click(x=1299, y=333)
            await worker_sleep(10)
            # 22 - 25
            max_attempts = 80
            minimum_checks = 4
            checks = 0
            for _ in range(max_attempts):
                if app.window(title="Confirm").exists():
                    checks = 0
                    await emsys.verify_warning_and_error("Confirm", "&Yes")
                    await emsys.verify_warning_and_error("Confirm", "Yes")
                    try:
                        pre_venda_window.wait("ready", timeout=1000)
                    except:
                        console.print("timeout")
                if app.window(title="Warning").exists():
                    checks = 0
                    await emsys.verify_warning_and_error("Warning", "OK")
                    await emsys.verify_warning_and_error("Warning", "&OK")
                    try:
                        pre_venda_window.wait("ready", timeout=1000)
                    except:
                        console.print("timeout")
                if app.window(title="Information").exists():
                    checks = 0
                    await emsys.verify_warning_and_error("Information", "OK")
                    try:
                        pre_venda_window.wait("ready", timeout=1000)
                    except:
                        console.print("timeout")
                if app.window(title="Error").exists():
                    checks = 0
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Erro durante o processo geracao_aprovacao_pedidos, erro : {erro}",
                        status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                    )
                if checks >= minimum_checks:
                    break
                checks = checks + 1
                await worker_sleep(2)
            
            await worker_sleep(10)
            console.print("Clicando em aprovar venda...")
            pyautogui.click(x=1310, y=352)
            await worker_sleep(30)

            max_attempts = 80
            minimum_checks = 4
            checks = 0
            for _ in range(max_attempts):
                if app.window(title="Confirm").exists():
                    checks = 0
                    await emsys.verify_warning_and_error("Confirm", "&Yes")
                    try:
                        pre_venda_window.wait("ready", timeout=1000)
                    except:
                        console.print("timeout")
                if app.window(title="Warning").exists():
                    checks = 0
                    await emsys.verify_warning_and_error("Warning", "OK")
                    try:
                        pre_venda_window.wait("ready", timeout=1000)
                    except:
                        console.print("timeout")
                if app.window(title="Information").exists():
                    checks = 0
                    await emsys.verify_warning_and_error("Information", "OK")
                    try:
                        pre_venda_window.wait("ready", timeout=1000)
                    except:
                        console.print("timeout")
                if app.window(title="Error").exists():
                    checks = 0
                    return RpaRetornoProcessoDTO(
                        sucesso=False,
                        retorno=f"Erro durante o processo geracao_aprovacao_pedidos, erro : {erro}",
                        status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
                    )
                if checks >= minimum_checks:
                    break
                checks = checks + 1
                await worker_sleep(2)
            console.print("Aguardando EMSys sincronizar... ( 15 minutos )")
            await worker_sleep(60*15)
            return RpaRetornoProcessoDTO(
                sucesso=True,
                retorno="Pre venda lancada com sucesso",
                status=RpaHistoricoStatusEnum.Sucesso
            )
    except Exception as erro:
        console.print(erro)
        return RpaRetornoProcessoDTO(
            sucesso=False,
            retorno=f"Erro durante o processo geracao_aprovacao_pedidos, erro : {erro}",
            status=RpaHistoricoStatusEnum.Falha, tags=[RpaTagDTO(descricao=RpaTagEnum.Tecnico)]
        )




from datetime import datetime
from enum import Enum
from typing import Optional, List

from pydantic import BaseModel, Field

class RpaHistoricoStatusEnum(str, Enum):
    Descartado = "D"
    Falha = "F"
    Processando = "P"
    Reservado = "R"
    Sucesso = "S"

class RpaTagEnum(str, Enum):
    Negocio = "NEGÓCIO"
    Tecnico = "TÉCNICO"

class RpaTagDTO(BaseModel):
    uuidTagRpa: Optional[str] = Field(None, alias="uuidTagRpa")
    descricao: RpaTagEnum = Field(..., alias="descricao")


    class Config:
        populate_by_name = True


class RpaRetornoProcessoDTO(BaseModel):
    sucesso: bool = Field(..., alias="sucesso")
    retorno: str = Field(..., alias="retorno")
    status: RpaHistoricoStatusEnum = Field(..., alias="status")
    tags: Optional[List[RpaTagDTO]] = Field(None, alias="tags")


    class Config:
        populate_by_name = True



class RpaHistoricoRequestDTO(BaseModel):
    uuidHistorico: Optional[str] = Field(None, alias="uuidHistorico")
    uuidProcesso: str = Field(..., alias="uuidProcesso")
    uuidRobo: Optional[str] = Field(None, alias="uuidRobo")
    prioridade: int
    desStatus: RpaHistoricoStatusEnum = Field(..., alias="desStatus")
    configEntrada: Optional[dict] = Field(None, alias="configEntrada")
    retorno: Optional[RpaRetornoProcessoDTO] = Field(None, alias="retorno")
    datEntradaFila: Optional[datetime] = Field(None, alias="datEntradaFila")
    datInicioExecucao: Optional[datetime] = Field(None, alias="datInicioExecucao")
    datFimExecucao: Optional[datetime] = Field(None, alias="datFimExecucao")
    identificador: Optional[str] = Field(None, alias="identificador")
    tags: Optional[List[RpaTagDTO]] = Field(None, alias="tags")

    class Config:
        populate_by_name = True
